@@ -9,26 +9,15 @@
 #include <obs-frontend-api.h>
 
 #include "advanced-scene-switcher.hpp"
+#include "switcher-data.hpp"
 #include "status-control.hpp"
+#include "scene-switch-helpers.hpp"
 #include "curl-helper.hpp"
+#include "platform-funcs.hpp"
 #include "utility.hpp"
 #include "version.h"
 
-const char *obs_module_text(const char *text)
-{
-	return switcher->translate(text);
-}
-
-obs_module_t *obs_current_module()
-{
-	return switcher->modulePtr;
-}
-
-SwitcherData *switcher = nullptr;
-SwitcherData *GetSwitcher()
-{
-	return switcher;
-}
+namespace advss {
 
 AdvSceneSwitcher *AdvSceneSwitcher::window = nullptr;
 
@@ -40,11 +29,9 @@ AdvSceneSwitcher::AdvSceneSwitcher(QWidget *parent)
 {
 	switcher->settingsWindowOpened = true;
 	ui->setupUi(this);
-
 	std::lock_guard<std::mutex> lock(switcher->m);
-
 	switcher->Prune();
-	loadUI();
+	LoadUI();
 }
 
 AdvSceneSwitcher::~AdvSceneSwitcher()
@@ -55,51 +42,76 @@ AdvSceneSwitcher::~AdvSceneSwitcher()
 	}
 }
 
-bool translationAvailable()
+static bool translationAvailable()
 {
 	return !!strcmp(obs_module_text("AdvSceneSwitcher.pluginName"),
 			"AdvSceneSwitcher.pluginName");
 }
 
-void AdvSceneSwitcher::loadUI()
+static void DisplayMissingDependencyWarning()
 {
-	if (!translationAvailable()) {
-		QString msg = "Failed to find plug-in's 'data' directory.\n"
-			      "Please check installation instructions!\n\n"
-			      "Data most likely expected at:\n\n";
-#ifdef _WIN32
-		msg += QString::fromStdString(
-			(std::filesystem::current_path().string()));
-		msg += "/";
-#endif
-		msg += obs_get_module_data_path(obs_current_module());
-		(void)DisplayMessage(msg);
+	if (!switcher->warnPluginLoadFailure ||
+	    switcher->loadFailureLibs.isEmpty()) {
+		return;
 	}
 
-	setupGeneralTab();
-	setupTitleTab();
-	setupExecutableTab();
-	setupRegionTab();
-	setupPauseTab();
-	setupSequenceTab();
-	setupTransitionsTab();
-	setupIdleTab();
-	setupRandomTab();
-	setupMediaTab();
-	setupFileTab();
-	setupTimeTab();
-	setupAudioTab();
-	setupVideoTab();
-	setupNetworkTab();
-	setupSceneGroupTab();
-	setupTriggerTab();
-	setupMacroTab();
+	QString failedLibsString = "<ul>";
+	for (const auto &lib : switcher->loadFailureLibs) {
+		failedLibsString += "<li>" + lib + "</li>";
+	}
+	failedLibsString += "</ul>";
+	QString warning(obs_module_text(
+		"AdvSceneSwitcher.generalTab.generalBehavior.warnPluginLoadFailureMessage"));
+	DisplayMessage(warning.arg(failedLibsString));
+}
 
-	setDeprecationWarnings();
-	setTabOrder();
-	setCurrentTab();
-	restoreWindowGeo();
-	checkFirstTimeSetup();
+static void DisplayMissingDataDirWarning()
+{
+	if (translationAvailable()) {
+		return;
+	}
+
+	QString msg = "Failed to find plug-in's 'data' directory.\n"
+		      "Please check installation instructions!\n\n"
+		      "Data most likely expected at:\n\n";
+#ifdef _WIN32
+	msg += QString::fromStdString(
+		(std::filesystem::current_path().string()));
+	msg += "/";
+#endif
+	msg += obs_get_module_data_path(obs_current_module());
+	DisplayMessage(msg);
+}
+
+void AdvSceneSwitcher::LoadUI()
+{
+	DisplayMissingDataDirWarning();
+	DisplayMissingDependencyWarning();
+
+	SetupGeneralTab();
+	SetupTitleTab();
+	SetupExecutableTab();
+	SetupRegionTab();
+	SetupPauseTab();
+	SetupSequenceTab();
+	SetupTransitionsTab();
+	SetupIdleTab();
+	SetupRandomTab();
+	SetupMediaTab();
+	SetupFileTab();
+	SetupTimeTab();
+	SetupAudioTab();
+	SetupVideoTab();
+	SetupNetworkTab();
+	SetupSceneGroupTab();
+	SetupTriggerTab();
+	SetupMacroTab();
+
+	SetDeprecationWarnings();
+	SetTabOrder();
+	SetCurrentTab();
+	RestoreWindowGeo();
+	CheckFirstTimeSetup();
 
 	loading = false;
 }
@@ -107,13 +119,15 @@ void AdvSceneSwitcher::loadUI()
 /******************************************************************************
  * Saving and loading
  ******************************************************************************/
+static void AskForBackup(obs_data_t *obj);
+
 static void SaveSceneSwitcher(obs_data_t *save_data, bool saving, void *)
 {
 	if (saving) {
 		std::lock_guard<std::mutex> lock(switcher->m);
 		switcher->Prune();
 		obs_data_t *obj = obs_data_create();
-		switcher->saveSettings(obj);
+		switcher->SaveSettings(obj);
 		obs_data_set_obj(save_data, "advanced-scene-switcher", obj);
 		obs_data_release(obj);
 	} else {
@@ -127,10 +141,10 @@ static void SaveSceneSwitcher(obs_data_t *save_data, bool saving, void *)
 		if (!obj) {
 			obj = obs_data_create();
 		}
-		if (switcher->versionChanged(obj, g_GIT_SHA1)) {
+		if (switcher->VersionChanged(obj, g_GIT_SHA1)) {
 			AskForBackup(obj);
 		}
-		switcher->loadSettings(obj);
+		switcher->LoadSettings(obj);
 		obs_data_release(obj);
 		switcher->m.unlock();
 
@@ -140,6 +154,34 @@ static void SaveSceneSwitcher(obs_data_t *save_data, bool saving, void *)
 	}
 }
 
+static void AskForBackup(obs_data_t *obj)
+{
+	bool backupSettings = DisplayMessage(
+		obs_module_text("AdvSceneSwitcher.askBackup"), true);
+
+	if (!backupSettings) {
+		return;
+	}
+
+	QString path = QFileDialog::getSaveFileName(
+		nullptr,
+		obs_module_text(
+			"AdvSceneSwitcher.generalTab.saveOrLoadsettings.exportWindowTitle"),
+		GetDefaultSettingsSaveLocation(),
+		obs_module_text(
+			"AdvSceneSwitcher.generalTab.saveOrLoadsettings.textType"));
+	if (path.isEmpty()) {
+		return;
+	}
+
+	QFile file(path);
+	if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
+		return;
+	}
+
+	obs_data_save_json(obj, file.fileName().toUtf8().constData());
+}
+
 /******************************************************************************
  * Main switcher thread
  ******************************************************************************/
@@ -183,7 +225,7 @@ void SwitcherData::Thread()
 		}
 
 		vblog(LOG_INFO, "try to sleep for %ld", duration.count());
-		setWaitScene();
+		SetWaitScene();
 		cv.wait_for(lock, duration);
 
 		startTime = std::chrono::high_resolution_clock::now();
@@ -197,13 +239,13 @@ void SwitcherData::Thread()
 		if (checkPause()) {
 			continue;
 		}
-		setPreconditions();
-		match = checkForMatch(scene, transition, linger,
+		SetPreconditions();
+		match = CheckForMatch(scene, transition, linger,
 				      setPrevSceneAfterLinger, macroMatch);
 		if (stop) {
 			break;
 		}
-		checkNoMatchSwitch(match, scene, transition, sleep);
+		CheckNoMatchSwitch(match, scene, transition, sleep);
 		checkSwitchCooldown(match);
 
 		if (linger) {
@@ -211,14 +253,14 @@ void SwitcherData::Thread()
 			vblog(LOG_INFO, "sleep for %ld before switching scene",
 			      duration.count());
 
-			setWaitScene();
+			SetWaitScene();
 			cv.wait_for(lock, duration);
 
 			if (stop) {
 				break;
 			}
 
-			if (sceneChangedDuringWait()) {
+			if (SceneChangedDuringWait()) {
 				vblog(LOG_INFO,
 				      "scene was changed manually - ignoring match");
 
@@ -229,13 +271,13 @@ void SwitcherData::Thread()
 			}
 		}
 
-		ClearWebsocketMessages();
+		ResetForNextInterval();
 
 		if (match) {
 			if (macroMatch) {
-				runMacros();
+				RunMacros();
 			} else {
-				switchScene({scene, transition, 0});
+				SwitchScene({scene, transition, 0});
 			}
 		}
 
@@ -248,7 +290,7 @@ void SwitcherData::Thread()
 	blog(LOG_INFO, "stopped");
 }
 
-void SwitcherData::setPreconditions()
+void SwitcherData::SetPreconditions()
 {
 	// Window title
 	lastTitle = currentTitle;
@@ -276,13 +318,23 @@ void SwitcherData::setPreconditions()
 	GetForegroundProcessName(currentForegroundProcess);
 
 	// Cursor
-	std::pair<int, int> cursorPos = getCursorPos();
+	std::pair<int, int> cursorPos = GetCursorPos();
 	cursorPosChanged = cursorPos.first != switcher->lastCursorPos.first ||
 			   cursorPos.second != switcher->lastCursorPos.second;
-	lastCursorPos = getCursorPos();
+	lastCursorPos = GetCursorPos();
 }
 
-bool SwitcherData::checkForMatch(OBSWeakSource &scene,
+void SwitcherData::ResetForNextInterval()
+{
+	// Core reset functions
+	ClearWebsocketMessages();
+	// Plugin reset functions
+	for (const auto &func : resetForNextIntervalFuncs) {
+		func();
+	}
+}
+
+bool SwitcherData::CheckForMatch(OBSWeakSource &scene,
 				 OBSWeakSource &transition, int &linger,
 				 bool &setPrevSceneAfterLinger,
 				 bool &macroMatch)
@@ -332,7 +384,7 @@ bool SwitcherData::checkForMatch(OBSWeakSource &scene,
 			match = checkVideoSwitch(scene, transition);
 			break;
 		case macro_func:
-			if (checkMacros()) {
+			if (CheckMacros()) {
 				match = true;
 				macroMatch = true;
 			}
@@ -349,48 +401,20 @@ bool SwitcherData::checkForMatch(OBSWeakSource &scene,
 	return match;
 }
 
-void switchScene(const sceneSwitchInfo &sceneSwitch, bool force)
+static void ResetMacros()
 {
-	if (!sceneSwitch.scene && switcher->verbose) {
-		blog(LOG_INFO, "nothing to switch to");
-		return;
+	for (auto &m : switcher->macros) {
+		m->ResetRunCount();
+		m->ResetTimers();
 	}
-
-	obs_source_t *source = obs_weak_source_get_source(sceneSwitch.scene);
-	obs_source_t *currentSource = obs_frontend_get_current_scene();
-
-	if (source && (source != currentSource || force)) {
-		transitionData currentTransitionData;
-		setNextTransition(sceneSwitch, currentSource,
-				  currentTransitionData);
-		obs_frontend_set_current_scene(source);
-		if (switcher->transitionOverrideOverride) {
-			restoreTransitionOverride(source,
-						  currentTransitionData);
-		}
-
-		if (switcher->verbose) {
-			blog(LOG_INFO, "switched scene");
-		}
-
-		if (switcher->networkConfig.ShouldSendSceneChange()) {
-			switcher->server.sendMessage(sceneSwitch);
-		}
-	}
-	obs_source_release(currentSource);
-	obs_source_release(source);
-}
-
-void switchPreviewScene(const OBSWeakSource &ws)
-{
-	auto source = obs_weak_source_get_source(ws);
-	obs_frontend_set_current_preview_scene(source);
-	obs_source_release(source);
 }
 
 void SwitcherData::Start()
 {
 	if (!(th && th->isRunning())) {
+		ResetForNextInterval();
+		ResetMacros();
+
 		stop = false;
 		th = new SwitcherThread();
 		th->start((QThread::Priority)threadPriority);
@@ -415,14 +439,6 @@ void SwitcherData::Start()
 	}
 }
 
-void ResetMacros()
-{
-	for (auto &m : switcher->macros) {
-		m->ResetRunCount();
-		m->ResetTimers();
-	}
-}
-
 void SwitcherData::Stop()
 {
 	if (th && th->isRunning()) {
@@ -436,7 +452,6 @@ void SwitcherData::Stop()
 		th = nullptr;
 
 		writeToStatusFile("Advanced Scene Switcher stopped");
-		ResetMacros();
 	}
 
 	server.stop();
@@ -449,13 +464,13 @@ void SwitcherData::Stop()
 	}
 }
 
-void SwitcherData::setWaitScene()
+void SwitcherData::SetWaitScene()
 {
 	waitScene = obs_frontend_get_current_scene();
 	obs_source_release(waitScene);
 }
 
-bool SwitcherData::sceneChangedDuringWait()
+bool SwitcherData::SceneChangedDuringWait()
 {
 	obs_source_t *currentSource = obs_frontend_get_current_scene();
 	if (!currentSource) {
@@ -468,7 +483,7 @@ bool SwitcherData::sceneChangedDuringWait()
 // Relies on the fact that switcher->currentScene will only be updated on event
 // OBS_FRONTEND_EVENT_SCENE_CHANGED but obs_frontend_get_current_scene() will
 // already return the scene to be transitioned to.
-bool SwitcherData::anySceneTransitionStarted()
+bool SwitcherData::AnySceneTransitionStarted()
 {
 	auto currentSceneSrouce = obs_frontend_get_current_scene();
 	auto currentScene = obs_source_get_weak_source(currentSceneSrouce);
@@ -489,13 +504,13 @@ extern "C" void FreeSceneSwitcher()
 	switcher = nullptr;
 }
 
-void handleSceneChange()
+static void handleSceneChange()
 {
 	switcher->lastSceneChangeTime =
 		std::chrono::high_resolution_clock::now();
 
 	// Stop waiting if scene was changed
-	if (switcher->sceneChangedDuringWait()) {
+	if (switcher->SceneChangedDuringWait()) {
 		switcher->cv.notify_one();
 	}
 
@@ -523,31 +538,33 @@ void handleSceneChange()
 	}
 }
 
-void setLiveTime()
+static void setLiveTime()
 {
 	switcher->liveTime = QDateTime::currentDateTime();
 }
 
-void resetLiveTime()
+static void resetLiveTime()
 {
 	switcher->liveTime = QDateTime();
 }
 
-void checkAutoStartRecording()
+static void checkAutoStartRecording()
 {
-	if (switcher->autoStartEvent == AutoStartEvent::RECORDING ||
-	    switcher->autoStartEvent == AutoStartEvent::RECORINDG_OR_STREAMING)
+	if (switcher->autoStartEvent == SwitcherData::AutoStart::RECORDING ||
+	    switcher->autoStartEvent ==
+		    SwitcherData::AutoStart::RECORINDG_OR_STREAMING)
 		switcher->Start();
 }
 
-void checkAutoStartStreaming()
+static void checkAutoStartStreaming()
 {
-	if (switcher->autoStartEvent == AutoStartEvent::STREAMING ||
-	    switcher->autoStartEvent == AutoStartEvent::RECORINDG_OR_STREAMING)
+	if (switcher->autoStartEvent == SwitcherData::AutoStart::STREAMING ||
+	    switcher->autoStartEvent ==
+		    SwitcherData::AutoStart::RECORINDG_OR_STREAMING)
 		switcher->Start();
 }
 
-void handlePeviewSceneChange()
+static void handlePeviewSceneChange()
 {
 	if (switcher->networkConfig.ShouldSendPrviewSceneChange()) {
 		auto source = obs_frontend_get_current_preview_scene();
@@ -558,43 +575,45 @@ void handlePeviewSceneChange()
 	}
 }
 
-void setReplayBufferSaved()
+static void setReplayBufferSaved()
 {
 	switcher->replayBufferSaved = true;
 }
 
-void setTranstionEnd()
+static void setTranstionEnd()
 {
 	switcher->lastTransitionEndTime =
 		std::chrono::high_resolution_clock::now();
 	switcher->macroTransitionCv.notify_all();
 }
 
-void setStreamStarting()
+static void setStreamStarting()
 {
 	switcher->lastStreamStartingTime =
 		std::chrono::high_resolution_clock::now();
 }
 
-void setStreamStopping()
+static void setStreamStopping()
 {
 	switcher->lastStreamStoppingTime =
 		std::chrono::high_resolution_clock::now();
 }
 
-void handleExit()
+static void handleExit()
 {
+	if (!switcher) {
+		return;
+	}
 	switcher->obsIsShuttingDown = true;
-	if (switcher && switcher->obsIsShuttingDown &&
-	    switcher->shutdownConditionCount) {
+	if (switcher->shutdownConditionCount) {
 		switcher->Stop();
-		switcher->checkMacros();
-		switcher->runMacros();
+		switcher->CheckMacros();
+		switcher->RunMacros();
 	}
 	FreeSceneSwitcher();
 }
 
-void handleSceneCollectionChanging()
+static void handleSceneCollectionChanging()
 {
 	if (switcher->settingsWindowOpened) {
 		AdvSceneSwitcher::window->close();
@@ -660,7 +679,7 @@ static void OBSEvent(enum obs_frontend_event event, void *switcher)
 	}
 }
 
-void LoadPlugins()
+static void LoadPlugins()
 {
 	QFileInfo libPath(
 		QString(obs_get_module_binary_path(obs_current_module())));
@@ -684,6 +703,7 @@ void LoadPlugins()
 			blog(LOG_WARNING, "failed to load \"%s\": %s",
 			     file.toStdString().c_str(),
 			     lib->errorString().toStdString().c_str());
+			switcher->loadFailureLibs << file;
 		}
 	}
 }
@@ -703,14 +723,12 @@ void OpenSettingsWindow()
 	}
 }
 
-extern "C" void InitSceneSwitcher(obs_module_t *m, translateFunc t)
+extern "C" void InitSceneSwitcher(obs_module_t *module, translateFunc translate)
 {
 	blog(LOG_INFO, "version: %s", g_GIT_TAG);
 	blog(LOG_INFO, "version: %s", g_GIT_SHA1);
 
-	switcher = new SwitcherData;
-	switcher->modulePtr = m;
-	switcher->translate = t;
+	switcher = new SwitcherData(module, translate);
 
 	PlatformInit();
 	LoadPlugins();
@@ -723,3 +741,5 @@ extern "C" void InitSceneSwitcher(obs_module_t *m, translateFunc t)
 		obs_module_text("AdvSceneSwitcher.pluginName"));
 	action->connect(action, &QAction::triggered, OpenSettingsWindow);
 }
+
+} // namespace advss
