@@ -1,9 +1,12 @@
 #include "advanced-scene-switcher.hpp"
+#include "switcher-data.hpp"
 #include "macro-action-edit.hpp"
 #include "macro-action-scene-switch.hpp"
 #include "section.hpp"
 #include "utility.hpp"
 
+namespace advss {
+
 std::map<std::string, MacroActionInfo> &MacroActionFactory::GetMap()
 {
 	static std::map<std::string, MacroActionInfo> _methods;
@@ -154,7 +157,7 @@ std::shared_ptr<MacroSegment> MacroActionEdit::Data()
 
 void AdvSceneSwitcher::AddMacroAction(int idx)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -193,7 +196,7 @@ void AdvSceneSwitcher::AddMacroAction(int idx)
 
 void AdvSceneSwitcher::on_actionAdd_clicked()
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -211,7 +214,7 @@ void AdvSceneSwitcher::on_actionAdd_clicked()
 
 void AdvSceneSwitcher::RemoveMacroAction(int idx)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -237,7 +240,7 @@ void AdvSceneSwitcher::RemoveMacroAction(int idx)
 void AdvSceneSwitcher::on_actionRemove_clicked()
 {
 	if (currentActionIdx == -1) {
-		auto macro = getSelectedMacro();
+		auto macro = GetSelectedMacro();
 		if (!macro) {
 			return;
 		}
@@ -289,7 +292,7 @@ void AdvSceneSwitcher::SwapActions(Macro *m, int pos1, int pos2)
 
 void AdvSceneSwitcher::MoveMacroActionUp(int idx)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -304,7 +307,7 @@ void AdvSceneSwitcher::MoveMacroActionUp(int idx)
 
 void AdvSceneSwitcher::MoveMacroActionDown(int idx)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -319,7 +322,7 @@ void AdvSceneSwitcher::MoveMacroActionDown(int idx)
 
 void AdvSceneSwitcher::MacroActionSelectionChanged(int idx)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -339,7 +342,7 @@ void AdvSceneSwitcher::MacroActionSelectionChanged(int idx)
 
 void AdvSceneSwitcher::MacroActionReorder(int to, int from)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -361,3 +364,5 @@ void AdvSceneSwitcher::MacroActionReorder(int to, int from)
 	HighlightAction(to);
 	emit(MacroSegmentOrderChanged());
 }
+
+} // namespace advss
