@@ -1,8 +1,10 @@
 #include "macro-action-http.hpp"
-#include "advanced-scene-switcher.hpp"
+#include "switcher-data.hpp"
 #include "utility.hpp"
 #include "curl-helper.hpp"
 
+namespace advss {
+
 const std::string MacroActionHttp::id = "http";
 
 bool MacroActionHttp::_registered = MacroActionFactory::Register(
@@ -16,22 +18,37 @@ const static std::map<MacroActionHttp::Method, std::string> methods = {
 	 "AdvSceneSwitcher.action.http.type.post"},
 };
 
-size_t DropCB(void *, size_t size, size_t nmemb, std::string *)
+static size_t DropCB(void *, size_t size, size_t nmemb, std::string *)
 {
 	return size * nmemb;
 }
 
-size_t WriteCB(void *ptr, size_t size, size_t nmemb, std::string *buffer)
+static size_t WriteCB(void *ptr, size_t size, size_t nmemb, std::string *buffer)
 {
 	buffer->append((char *)ptr, nmemb);
 	return size * nmemb;
 }
 
+void MacroActionHttp::SetupHeaders()
+{
+	if (!_setHeaders) {
+		return;
+	}
+	struct curl_slist *headers = nullptr;
+	for (auto &header : _headers) {
+		headers = switcher->curl.SlistAppend(headers, header.c_str());
+	}
+	if (!_headers.empty()) {
+		switcher->curl.SetOpt(CURLOPT_HTTPHEADER, headers);
+	}
+}
+
 void MacroActionHttp::Get()
 {
 	switcher->curl.SetOpt(CURLOPT_URL, _url.c_str());
 	switcher->curl.SetOpt(CURLOPT_HTTPGET, 1L);
-	switcher->curl.SetOpt(CURLOPT_TIMEOUT_MS, _timeout.seconds * 1000);
+	switcher->curl.SetOpt(CURLOPT_TIMEOUT_MS, _timeout.Milliseconds());
+	SetupHeaders();
 
 	std::string response;
 	if (IsReferencedInVars()) {
@@ -49,7 +66,8 @@ void MacroActionHttp::Post()
 {
 	switcher->curl.SetOpt(CURLOPT_URL, _url.c_str());
 	switcher->curl.SetOpt(CURLOPT_POSTFIELDS, _data.c_str());
-	switcher->curl.SetOpt(CURLOPT_TIMEOUT_MS, _timeout.seconds * 1000);
+	switcher->curl.SetOpt(CURLOPT_TIMEOUT_MS, _timeout.Milliseconds());
+	SetupHeaders();
 	switcher->curl.Perform();
 }
 
@@ -90,8 +108,10 @@ void MacroActionHttp::LogAction() const
 bool MacroActionHttp::Save(obs_data_t *obj) const
 {
 	MacroAction::Save(obj);
-	obs_data_set_string(obj, "url", _url.c_str());
+	_url.Save(obj, "url");
 	_data.Save(obj, "data");
+	obs_data_set_bool(obj, "setHeaders", _setHeaders);
+	_headers.Save(obj, "headers", "header");
 	obs_data_set_int(obj, "method", static_cast<int>(_method));
 	_timeout.Save(obj);
 	return true;
@@ -100,8 +120,10 @@ bool MacroActionHttp::Save(obs_data_t *obj) const
 bool MacroActionHttp::Load(obs_data_t *obj)
 {
 	MacroAction::Load(obj);
-	_url = obs_data_get_string(obj, "url");
+	_url.Load(obj, "url");
 	_data.Load(obj, "data");
+	_setHeaders = obs_data_get_bool(obj, "setHeaders");
+	_headers.Load(obj, "headers", "header");
 	_method = static_cast<Method>(obs_data_get_int(obj, "method"));
 	_timeout.Load(obj);
 	return true;
@@ -109,7 +131,7 @@ bool MacroActionHttp::Load(obs_data_t *obj)
 
 std::string MacroActionHttp::GetShortDesc() const
 {
-	return _url;
+	return _url.UnresolvedValue();
 }
 
 static inline void populateMethodSelection(QComboBox *list)
@@ -122,9 +144,15 @@ static inline void populateMethodSelection(QComboBox *list)
 MacroActionHttpEdit::MacroActionHttpEdit(
 	QWidget *parent, std::shared_ptr<MacroActionHttp> entryData)
 	: QWidget(parent),
-	  _url(new QLineEdit()),
+	  _url(new VariableLineEdit(this)),
 	  _methods(new QComboBox()),
 	  _data(new VariableTextEdit(this)),
+	  _setHeaders(new QCheckBox(
+		  obs_module_text("AdvSceneSwitcher.action.http.setHeaders"))),
+	  _headerListLayout(new QVBoxLayout()),
+	  _headerList(new StringListEdit(
+		  this, obs_module_text("AdvSceneSwitcher.action.http.headers"),
+		  obs_module_text("AdvSceneSwitcher.action.http.addHeader"))),
 	  _timeout(new DurationSelection(this, false))
 {
 	populateMethodSelection(_methods);
@@ -135,8 +163,13 @@ MacroActionHttpEdit::MacroActionHttpEdit(
 			 SLOT(DataChanged()));
 	QWidget::connect(_methods, SIGNAL(currentIndexChanged(int)), this,
 			 SLOT(MethodChanged(int)));
-	QWidget::connect(_timeout, SIGNAL(DurationChanged(double)), this,
-			 SLOT(TimeoutChanged(double)));
+	QWidget::connect(_setHeaders, SIGNAL(stateChanged(int)), this,
+			 SLOT(SetHeadersChanged(int)));
+	QWidget::connect(_headerList,
+			 SIGNAL(StringListChanged(const StringList &)), this,
+			 SLOT(HeadersChanged(const StringList &)));
+	QWidget::connect(_timeout, SIGNAL(DurationChanged(const Duration &)),
+			 this, SLOT(TimeoutChanged(const Duration &)));
 
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{url}}", _url},
@@ -145,16 +178,22 @@ MacroActionHttpEdit::MacroActionHttpEdit(
 		{"{{timeout}}", _timeout},
 	};
 	auto *actionLayout = new QHBoxLayout;
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text("AdvSceneSwitcher.action.http.entry.line1"),
 		actionLayout, widgetPlaceholders);
 	auto *timeoutLayout = new QHBoxLayout;
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text("AdvSceneSwitcher.action.http.entry.line2"),
 		timeoutLayout, widgetPlaceholders);
 
+	_headerListLayout->addWidget(new QLabel(
+		obs_module_text("AdvSceneSwitcher.action.http.headers")));
+	_headerListLayout->addWidget(_headerList);
+
 	QVBoxLayout *mainLayout = new QVBoxLayout;
 	mainLayout->addLayout(actionLayout);
+	mainLayout->addWidget(_setHeaders);
+	mainLayout->addLayout(_headerListLayout);
 	mainLayout->addWidget(_data);
 	mainLayout->addLayout(timeoutLayout);
 	setLayout(mainLayout);
@@ -170,8 +209,10 @@ void MacroActionHttpEdit::UpdateEntryData()
 		return;
 	}
 
-	_url->setText(QString::fromStdString(_entryData->_url));
+	_url->setText(_entryData->_url);
 	_data->setPlainText(_entryData->_data);
+	_setHeaders->setChecked(_entryData->_setHeaders);
+	_headerList->SetStringList(_entryData->_headers);
 	_methods->setCurrentIndex(static_cast<int>(_entryData->_method));
 	_timeout->SetDuration(_entryData->_timeout);
 	SetWidgetVisibility();
@@ -183,7 +224,7 @@ void MacroActionHttpEdit::URLChanged()
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_url = _url->text().toStdString();
 	emit(HeaderInfoChanged(_url->text()));
 }
@@ -194,33 +235,48 @@ void MacroActionHttpEdit::MethodChanged(int value)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_method = static_cast<MacroActionHttp::Method>(value);
 	SetWidgetVisibility();
 }
 
-void MacroActionHttpEdit::TimeoutChanged(double seconds)
+void MacroActionHttpEdit::TimeoutChanged(const Duration &dur)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_timeout.seconds = seconds;
+	auto lock = LockContext();
+	_entryData->_timeout = dur;
+}
+
+void MacroActionHttpEdit::SetHeadersChanged(int value)
+{
+	if (_loading || !_entryData) {
+		return;
+	}
+
+	auto lock = LockContext();
+	_entryData->_setHeaders = value;
+	SetWidgetVisibility();
+}
+
+void MacroActionHttpEdit::HeadersChanged(const StringList &headers)
+{
+	if (_loading || !_entryData) {
+		return;
+	}
+
+	auto lock = LockContext();
+	_entryData->_headers = headers;
+	adjustSize();
+	updateGeometry();
 }
 
 void MacroActionHttpEdit::SetWidgetVisibility()
 {
-	switch (_entryData->_method) {
-	case MacroActionHttp::Method::GET:
-		_data->hide();
-		break;
-	case MacroActionHttp::Method::POST:
-		_data->show();
-		break;
-	default:
-		break;
-	}
+	_data->setVisible(_entryData->_method == MacroActionHttp::Method::POST);
+	SetLayoutVisible(_headerListLayout, _entryData->_setHeaders);
 
 	adjustSize();
 	updateGeometry();
@@ -232,9 +288,11 @@ void MacroActionHttpEdit::DataChanged()
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_data = _data->toPlainText().toUtf8().constData();
 
 	adjustSize();
 	updateGeometry();
 }
+
+} // namespace advss
