@@ -4,6 +4,8 @@
 
 #include <QHBoxLayout>
 
+namespace advss {
+
 enum class PerformMacroAction {
 	PAUSE,
 	UNPAUSE,
@@ -51,7 +53,6 @@ public:
 
 private slots:
 	void MacroChanged(const QString &text);
-	void MacroRemove(const QString &name);
 	void ActionChanged(int value);
 signals:
 	void HeaderInfoChanged(const QString &);
@@ -65,3 +66,5 @@ private:
 	QHBoxLayout *_mainLayout;
 	bool _loading = true;
 };
+
+} // namespace advss
