@@ -1,7 +1,8 @@
 #include "macro-action-scene-order.hpp"
-#include "advanced-scene-switcher.hpp"
 #include "utility.hpp"
 
+namespace advss {
+
 const std::string MacroActionSceneOrder::id = "scene_order";
 
 bool MacroActionSceneOrder::_registered = MacroActionFactory::Register(
@@ -22,7 +23,7 @@ const static std::map<SceneOrderAction, std::string> actionTypes = {
 	 "AdvSceneSwitcher.action.sceneOrder.type.movePosition"},
 };
 
-void moveSceneItemsUp(std::vector<obs_scene_item *> &items)
+static void moveSceneItemsUp(std::vector<obs_scene_item *> &items)
 {
 	// In the case of the same source being in two sequential positions
 	// moving the sources up will cause the sources to swap positions due to
@@ -38,7 +39,7 @@ void moveSceneItemsUp(std::vector<obs_scene_item *> &items)
 	}
 }
 
-void moveSceneItemsDown(std::vector<obs_scene_item *> &items)
+static void moveSceneItemsDown(std::vector<obs_scene_item *> &items)
 {
 	for (auto &i : items) {
 		obs_sceneitem_set_order(i, OBS_ORDER_MOVE_DOWN);
@@ -46,7 +47,7 @@ void moveSceneItemsDown(std::vector<obs_scene_item *> &items)
 	}
 }
 
-void moveSceneItemsTop(std::vector<obs_scene_item *> &items)
+static void moveSceneItemsTop(std::vector<obs_scene_item *> &items)
 {
 	for (auto &i : items) {
 		obs_sceneitem_set_order(i, OBS_ORDER_MOVE_TOP);
@@ -54,7 +55,7 @@ void moveSceneItemsTop(std::vector<obs_scene_item *> &items)
 	}
 }
 
-void moveSceneItemsBottom(std::vector<obs_scene_item *> &items)
+static void moveSceneItemsBottom(std::vector<obs_scene_item *> &items)
 {
 	for (auto &i : items) {
 		obs_sceneitem_set_order(i, OBS_ORDER_MOVE_BOTTOM);
@@ -62,7 +63,7 @@ void moveSceneItemsBottom(std::vector<obs_scene_item *> &items)
 	}
 }
 
-void moveSceneItemsPos(std::vector<obs_scene_item *> &items, int pos)
+static void moveSceneItemsPos(std::vector<obs_scene_item *> &items, int pos)
 {
 	for (auto &i : items) {
 		obs_sceneitem_set_order_position(i, pos);
@@ -183,7 +184,7 @@ MacroActionSceneOrderEdit::MacroActionSceneOrderEdit(
 		{"{{actions}}", _actions},
 		{"{{position}}", _position},
 	};
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text("AdvSceneSwitcher.action.sceneOrder.entry"),
 		mainLayout, widgetPlaceholders);
 	setLayout(mainLayout);
@@ -213,7 +214,7 @@ void MacroActionSceneOrderEdit::SceneChanged(const SceneSelection &s)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_scene = s;
 }
 
@@ -223,7 +224,7 @@ void MacroActionSceneOrderEdit::SourceChanged(const SceneItemSelection &item)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_source = item;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -235,7 +236,7 @@ void MacroActionSceneOrderEdit::ActionChanged(int value)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_action = static_cast<SceneOrderAction>(value);
 	_position->setVisible(_entryData->_action ==
 			      SceneOrderAction::POSITION);
@@ -247,6 +248,8 @@ void MacroActionSceneOrderEdit::PositionChanged(int value)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_position = value;
 }
+
+} // namespace advss
