@@ -1,7 +1,10 @@
 #include "macro-action-scene-switch.hpp"
-#include "advanced-scene-switcher.hpp"
+#include "switcher-data.hpp"
+#include "scene-switch-helpers.hpp"
 #include "utility.hpp"
 
+namespace advss {
+
 using namespace std::chrono_literals;
 
 const std::string MacroActionSwitchScene::id = "scene_switch";
@@ -103,7 +106,7 @@ bool MacroActionSwitchScene::WaitForTransition(OBSWeakSource &scene,
 					       OBSWeakSource &transition)
 {
 	const int expectedTransitionDuration = getExpectedTransitionDuration(
-		scene, transition, _duration.seconds);
+		scene, transition, _duration.Seconds());
 	switcher->abortMacroWait = false;
 
 	std::unique_lock<std::mutex> lock(switcher->m);
@@ -121,7 +124,7 @@ bool MacroActionSwitchScene::PerformAction()
 {
 	auto scene = _scene.GetScene();
 	auto transition = _transition.GetTransition();
-	switchScene({scene, transition, (int)(_duration.seconds * 1000)},
+	SwitchScene({scene, transition, (int)(_duration.Milliseconds())},
 		    obs_frontend_preview_program_mode_active());
 	if (_blockUntilTransitionDone && scene) {
 		return WaitForTransition(scene, transition);
@@ -196,8 +199,8 @@ MacroActionSwitchSceneEdit::MacroActionSwitchSceneEdit(
 			 SIGNAL(TransitionChanged(const TransitionSelection &)),
 			 this,
 			 SLOT(TransitionChanged(const TransitionSelection &)));
-	QWidget::connect(_duration, SIGNAL(DurationChanged(double)), this,
-			 SLOT(DurationChanged(double)));
+	QWidget::connect(_duration, SIGNAL(DurationChanged(const Duration &)),
+			 this, SLOT(DurationChanged(const Duration &)));
 	QWidget::connect(_blockUntilTransitionDone, SIGNAL(stateChanged(int)),
 			 this, SLOT(BlockUntilTransitionDoneChanged(int)));
 
@@ -207,7 +210,7 @@ MacroActionSwitchSceneEdit::MacroActionSwitchSceneEdit(
 		{"{{duration}}", _duration},
 		{"{{blockUntilTransitionDone}}", _blockUntilTransitionDone},
 	};
-	placeWidgets(obs_module_text("AdvSceneSwitcher.action.scene.entry"),
+	PlaceWidgets(obs_module_text("AdvSceneSwitcher.action.scene.entry"),
 		     _entryLayout, widgetPlaceholders);
 
 	QVBoxLayout *mainLayout = new QVBoxLayout;
@@ -225,14 +228,14 @@ MacroActionSwitchSceneEdit::MacroActionSwitchSceneEdit(
 	_loading = false;
 }
 
-void MacroActionSwitchSceneEdit::DurationChanged(double seconds)
+void MacroActionSwitchSceneEdit::DurationChanged(const Duration &dur)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_duration.seconds = seconds;
+	auto lock = LockContext();
+	_entryData->_duration = dur;
 }
 
 void MacroActionSwitchSceneEdit::BlockUntilTransitionDoneChanged(int state)
@@ -241,7 +244,7 @@ void MacroActionSwitchSceneEdit::BlockUntilTransitionDoneChanged(int state)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_blockUntilTransitionDone = state;
 }
 
@@ -258,19 +261,19 @@ void MacroActionSwitchSceneEdit::SetDurationVisibility()
 	_entryLayout->removeWidget(_scenes);
 	_entryLayout->removeWidget(_transitions);
 	_entryLayout->removeWidget(_duration);
-	clearLayout(_entryLayout);
+	ClearLayout(_entryLayout);
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{scenes}}", _scenes},
 		{"{{transitions}}", _transitions},
 		{"{{duration}}", _duration},
 	};
 	if (fixedDuration) {
-		placeWidgets(
+		PlaceWidgets(
 			obs_module_text(
 				"AdvSceneSwitcher.action.scene.entry.noDuration"),
 			_entryLayout, widgetPlaceholders);
 	} else {
-		placeWidgets(
+		PlaceWidgets(
 			obs_module_text("AdvSceneSwitcher.action.scene.entry"),
 			_entryLayout, widgetPlaceholders);
 	}
@@ -282,7 +285,7 @@ void MacroActionSwitchSceneEdit::SceneChanged(const SceneSelection &s)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_scene = s;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -294,7 +297,9 @@ void MacroActionSwitchSceneEdit::TransitionChanged(const TransitionSelection &t)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_transition = t;
 	SetDurationVisibility();
 }
+
+} // namespace advss
