@@ -1,7 +1,8 @@
 #include "macro-action-scene-transform.hpp"
-#include "advanced-scene-switcher.hpp"
 #include "utility.hpp"
 
+namespace advss {
+
 const std::string MacroActionSceneTransform::id = "scene_transform";
 
 bool MacroActionSceneTransform::_registered = MacroActionFactory::Register(
@@ -59,7 +60,7 @@ bool MacroActionSceneTransform::Load(obs_data_t *obj)
 	// Convert old data format
 	// TODO: Remove in future version
 	if (!obs_data_has_user_value(obj, "settings")) {
-		loadTransformState(obj, _info, _crop);
+		LoadTransformState(obj, _info, _crop);
 		_settings = ConvertSettings();
 	}
 	return true;
@@ -76,7 +77,7 @@ std::string MacroActionSceneTransform::GetShortDesc() const
 std::string MacroActionSceneTransform::ConvertSettings()
 {
 	auto data = obs_data_create();
-	saveTransformState(data, _info, _crop);
+	SaveTransformState(data, _info, _crop);
 	std::string json = obs_data_get_json(data);
 	obs_data_release(data);
 	return json;
@@ -88,7 +89,7 @@ void MacroActionSceneTransform::ApplySettings(const std::string &settings)
 	if (!data) {
 		return;
 	}
-	loadTransformState(data, _info, _crop);
+	LoadTransformState(data, _info, _crop);
 	auto items = _source.GetSceneItems(_scene);
 	if (items.empty()) {
 		return;
@@ -142,7 +143,7 @@ MacroActionSceneTransformEdit::MacroActionSceneTransformEdit(
 		{"{{settings}}", _settings},
 		{"{{getSettings}}", _getSettings},
 	};
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text("AdvSceneSwitcher.action.sceneTransform.entry"),
 		entryLayout, widgetPlaceholders);
 
@@ -181,7 +182,7 @@ void MacroActionSceneTransformEdit::SceneChanged(const SceneSelection &s)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_scene = s;
 }
 
@@ -191,7 +192,7 @@ void MacroActionSceneTransformEdit::SourceChanged(const SceneItemSelection &item
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_source = item;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -208,8 +209,8 @@ void MacroActionSceneTransformEdit::GetSettingsClicked()
 		return;
 	}
 
-	auto settings = getSceneItemTransform(items[0]);
-	_settings->setPlainText(formatJsonString(settings));
+	auto settings = GetSceneItemTransform(items[0]);
+	_settings->setPlainText(FormatJsonString(settings));
 	for (auto item : items) {
 		obs_sceneitem_release(item);
 	}
@@ -221,9 +222,11 @@ void MacroActionSceneTransformEdit::SettingsChanged()
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_settings = _settings->toPlainText().toStdString();
 
 	adjustSize();
 	updateGeometry();
 }
+
+} // namespace advss
