@@ -1,7 +1,8 @@
 #include "macro-action-scene-visibility.hpp"
-#include "advanced-scene-switcher.hpp"
 #include "utility.hpp"
 
+namespace advss {
+
 const std::string MacroActionSceneVisibility::id = "scene_visibility";
 
 bool MacroActionSceneVisibility::_registered = MacroActionFactory::Register(
@@ -190,7 +191,7 @@ MacroActionSceneVisibilityEdit::MacroActionSceneVisibilityEdit(
 	_actions = new QComboBox();
 
 	populateSourceItemTypeSelection(_sourceTypes);
-	populateSourceGroupSelection(_sourceGroups);
+	PopulateSourceGroupSelection(_sourceGroups);
 	populateActionSelection(_actions);
 
 	QWidget::connect(_actions, SIGNAL(currentIndexChanged(int)), this,
@@ -214,7 +215,7 @@ MacroActionSceneVisibilityEdit::MacroActionSceneVisibilityEdit(
 		{"{{sources}}", _sources}, {"{{sourceGroups}}", _sourceGroups},
 		{"{{actions}}", _actions},
 	};
-	placeWidgets(obs_module_text(
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.action.sceneVisibility.entry"),
 		     mainLayout, widgetPlaceholders);
 	setLayout(mainLayout);
@@ -246,7 +247,7 @@ void MacroActionSceneVisibilityEdit::SceneChanged(const SceneSelection &s)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_scene = s;
 }
 
@@ -256,7 +257,7 @@ void MacroActionSceneVisibilityEdit::SourceTypeChanged(int value)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_sourceType = static_cast<SceneItemSourceType>(value);
 	SetWidgetVisibility();
 }
@@ -268,7 +269,7 @@ void MacroActionSceneVisibilityEdit::SourceChanged(
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_source = item;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -280,7 +281,7 @@ void MacroActionSceneVisibilityEdit::SourceGroupChanged(const QString &text)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	if (text == obs_module_text("AdvSceneSwitcher.selectItem")) {
 		_entryData->_sourceGroup = "";
 	} else {
@@ -296,7 +297,7 @@ void MacroActionSceneVisibilityEdit::ActionChanged(int value)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_action = static_cast<SceneVisibilityAction>(value);
 }
 
@@ -311,3 +312,5 @@ void MacroActionSceneVisibilityEdit::SetWidgetVisibility()
 				  SceneItemSourceType::SOURCE_GROUP);
 	adjustSize();
 }
+
+} // namespace advss
