@@ -1,8 +1,9 @@
 #include "macro-action-source.hpp"
-#include "advanced-scene-switcher.hpp"
 #include "utility.hpp"
 
-Q_DECLARE_METATYPE(SourceSettingButton);
+Q_DECLARE_METATYPE(advss::SourceSettingButton);
+
+namespace advss {
 
 const std::string MacroActionSource::id = "source";
 
@@ -85,7 +86,7 @@ bool MacroActionSource::PerformAction()
 		obs_source_set_enabled(s, false);
 		break;
 	case SourceAction::SETTINGS:
-		setSourceSettings(s, _settings);
+		SetSourceSettings(s, _settings);
 		break;
 	case SourceAction::REFRESH_SETTINGS:
 		refreshSourceSettings(s);
@@ -209,7 +210,7 @@ MacroActionSourceEdit::MacroActionSourceEdit(
 		{"{{getSettings}}", _getSettings},
 		{"{{settingsButtons}}", _settingsButtons},
 	};
-	placeWidgets(obs_module_text("AdvSceneSwitcher.action.source.entry"),
+	PlaceWidgets(obs_module_text("AdvSceneSwitcher.action.source.entry"),
 		     entryLayout, widgetPlaceholders);
 	mainLayout->addLayout(entryLayout);
 	mainLayout->addWidget(_warning);
@@ -250,7 +251,7 @@ void MacroActionSourceEdit::SourceChanged(const SourceSelection &source)
 	}
 
 	{
-		std::lock_guard<std::mutex> lock(switcher->m);
+		auto lock = LockContext();
 		_entryData->_source = source;
 	}
 	populateSourceButtonSelection(_settingsButtons,
@@ -265,7 +266,7 @@ void MacroActionSourceEdit::ActionChanged(int value)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_action = static_cast<SourceAction>(value);
 	SetWidgetVisibility();
 }
@@ -276,7 +277,7 @@ void MacroActionSourceEdit::ButtonChanged(int idx)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_button = qvariant_cast<SourceSettingButton>(
 		_settingsButtons->itemData(idx));
 }
@@ -287,8 +288,8 @@ void MacroActionSourceEdit::GetSettingsClicked()
 		return;
 	}
 
-	_settings->setPlainText(formatJsonString(
-		getSourceSettings(_entryData->_source.GetSource())));
+	_settings->setPlainText(FormatJsonString(
+		GetSourceSettings(_entryData->_source.GetSource())));
 }
 
 void MacroActionSourceEdit::SettingsChanged()
@@ -297,7 +298,7 @@ void MacroActionSourceEdit::SettingsChanged()
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_settings = _settings->toPlainText().toStdString();
 
 	adjustSize();
@@ -343,3 +344,5 @@ std::string SourceSettingButton::ToString() const
 	}
 	return "[" + id + "] " + description;
 }
+
+} // namespace advss
