@@ -8,6 +8,8 @@
 #include <QPushButton>
 #include <QComboBox>
 
+namespace advss {
+
 enum class SourceAction {
 	ENABLE,
 	DISABLE,
@@ -41,7 +43,7 @@ public:
 
 	SourceSelection _source;
 	SourceSettingButton _button;
-	VariableResolvingString _settings = "";
+	StringVariable _settings = "";
 	SourceAction _action = SourceAction::ENABLE;
 
 private:
@@ -87,3 +89,5 @@ private:
 	void SetWidgetVisibility();
 	bool _loading = true;
 };
+
+} // namespace advss
