@@ -1,15 +1,13 @@
 #pragma once
 #include "macro-action-edit.hpp"
+#include "variable-spinbox.hpp"
 
 #include <QDoubleSpinBox>
 #include <QComboBox>
 #include <QHBoxLayout>
 #include <chrono>
 
-enum class StreamAction {
-	STOP,
-	START,
-};
+namespace advss {
 
 class MacroActionStream : public MacroAction {
 public:
@@ -24,9 +22,16 @@ public:
 		return std::make_shared<MacroActionStream>(m);
 	}
 
-	StreamAction _action = StreamAction::STOP;
+	enum class Action {
+		STOP,
+		START,
+		KEYFRAME_INTERVAL,
+	};
+	Action _action = Action::STOP;
+	NumberVariable<int> _keyFrameInterval = 0;
 
 private:
+	void SetKeyFrameInterval();
 	bool CooldownDurationReached();
 	static std::chrono::high_resolution_clock::time_point s_lastAttempt;
 
@@ -52,12 +57,18 @@ public:
 
 private slots:
 	void ActionChanged(int value);
+	void KeyFrameIntervalChanged(const NumberVariable<int> &);
 
 protected:
 	QComboBox *_actions;
+	VariableSpinBox *_keyFrameInterval;
 	std::shared_ptr<MacroActionStream> _entryData;
 
 private:
+	void SetWidgetVisiblity();
+
 	QHBoxLayout *_mainLayout;
 	bool _loading = true;
 };
+
+} // namespace advss
