@@ -1,7 +1,8 @@
 #include "macro-action-transition.hpp"
-#include "advanced-scene-switcher.hpp"
 #include "utility.hpp"
 
+namespace advss {
+
 const std::string MacroActionTransition::id = "transition";
 
 bool MacroActionTransition::_registered = MacroActionFactory::Register(
@@ -29,7 +30,8 @@ void MacroActionTransition::SetSceneTransition()
 		obs_source_release(t);
 	}
 	if (_setDuration) {
-		obs_frontend_set_transition_duration(_duration.seconds * 1000);
+		obs_frontend_set_transition_duration(_duration.Seconds() *
+						     1000);
 	}
 }
 
@@ -43,7 +45,7 @@ void MacroActionTransition::SetTransitionOverride()
 	}
 	if (_setDuration) {
 		obs_data_set_int(data, "transition_duration",
-				 _duration.seconds * 1000);
+				 _duration.Milliseconds());
 	}
 	obs_data_release(data);
 	obs_source_release(scene);
@@ -51,8 +53,8 @@ void MacroActionTransition::SetTransitionOverride()
 
 #if (LIBOBS_API_VER >= MAKE_SEMANTIC_VERSION(27, 0, 0)) && \
 	(LIBOBS_API_VER < MAKE_SEMANTIC_VERSION(28, 0, 0))
-void obs_sceneitem_set_transition(obs_sceneitem_t *item, bool show,
-				  obs_source_t *transition)
+static void obs_sceneitem_set_transition(obs_sceneitem_t *item, bool show,
+					 obs_source_t *transition)
 {
 	if (show) {
 		obs_sceneitem_set_show_transition(item, transition);
@@ -61,8 +63,9 @@ void obs_sceneitem_set_transition(obs_sceneitem_t *item, bool show,
 	}
 }
 
-void obs_sceneitem_set_transition_duration(obs_sceneitem_t *item, bool show,
-					   uint32_t duration_ms)
+static void obs_sceneitem_set_transition_duration(obs_sceneitem_t *item,
+						  bool show,
+						  uint32_t duration_ms)
 {
 	if (show) {
 		obs_sceneitem_set_show_transition_duration(item, duration_ms);
@@ -91,7 +94,7 @@ void MacroActionTransition::SetSourceTransition(bool show)
 		}
 		if (_setDuration) {
 			obs_sceneitem_set_transition_duration(
-				item, show, _duration.seconds * 1000);
+				item, show, _duration.Milliseconds());
 		}
 		obs_sceneitem_release(item);
 	}
@@ -230,8 +233,8 @@ MacroActionTransitionEdit::MacroActionTransitionEdit(
 			 SIGNAL(TransitionChanged(const TransitionSelection &)),
 			 this,
 			 SLOT(TransitionChanged(const TransitionSelection &)));
-	QWidget::connect(_duration, SIGNAL(DurationChanged(double)), this,
-			 SLOT(DurationChanged(double)));
+	QWidget::connect(_duration, SIGNAL(DurationChanged(const Duration &)),
+			 this, SLOT(DurationChanged(const Duration &)));
 	QWidget::connect(_setTransition, SIGNAL(stateChanged(int)), this,
 			 SLOT(SetTransitionChanged(int)));
 	QWidget::connect(_setDuration, SIGNAL(stateChanged(int)), this,
@@ -247,13 +250,13 @@ MacroActionTransitionEdit::MacroActionTransitionEdit(
 		{"{{setDuration}}", _setDuration},
 	};
 	QHBoxLayout *typeLayout = new QHBoxLayout;
-	placeWidgets(obs_module_text(
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.action.transition.entry.line1"),
 		     typeLayout, widgetPlaceholders);
-	placeWidgets(obs_module_text(
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.action.transition.entry.line2"),
 		     _transitionLayout, widgetPlaceholders);
-	placeWidgets(obs_module_text(
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.action.transition.entry.line3"),
 		     _durationLayout, widgetPlaceholders);
 	QVBoxLayout *mainLayout = new QVBoxLayout;
@@ -291,7 +294,7 @@ void MacroActionTransitionEdit::SourceChanged(const SceneItemSelection &item)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_source = item;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -303,7 +306,7 @@ void MacroActionTransitionEdit::ActionChanged(int value)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_type = static_cast<MacroActionTransition::Type>(value);
 	SetWidgetVisibility();
 	emit HeaderInfoChanged(
@@ -316,7 +319,7 @@ void MacroActionTransitionEdit::SceneChanged(const SceneSelection &s)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_scene = s;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -328,20 +331,20 @@ void MacroActionTransitionEdit::TransitionChanged(const TransitionSelection &t)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_transition = t;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
 }
 
-void MacroActionTransitionEdit::DurationChanged(double seconds)
+void MacroActionTransitionEdit::DurationChanged(const Duration &dur)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_duration.seconds = seconds;
+	auto lock = LockContext();
+	_entryData->_duration = dur;
 }
 
 void MacroActionTransitionEdit::SetWidgetVisibility()
@@ -360,7 +363,7 @@ void MacroActionTransitionEdit::SetTransitionChanged(int state)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_setTransitionType = state;
 	_transitions->setEnabled(state);
 	if (state) {
@@ -377,7 +380,9 @@ void MacroActionTransitionEdit::SetDurationChanged(int state)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_setDuration = state;
 	_duration->setEnabled(state);
 }
+
+} // namespace advss
