@@ -1,8 +1,13 @@
 #include "macro-action-variable.hpp"
-#include "advanced-scene-switcher.hpp"
 #include "macro-condition-edit.hpp"
+#include "switcher-data.hpp"
+#include "advanced-scene-switcher.hpp"
+#include "macro.hpp"
+#include "math-helpers.hpp"
 #include "utility.hpp"
 
+namespace advss {
+
 const std::string MacroActionVariable::id = "variable";
 
 bool MacroActionVariable::_registered = MacroActionFactory::Register(
@@ -29,6 +34,10 @@ static std::map<MacroActionVariable::Type, std::string> actionTypes = {
 	 "AdvSceneSwitcher.action.variable.type.roundToInt"},
 	{MacroActionVariable::Type::SUBSTRING,
 	 "AdvSceneSwitcher.action.variable.type.subString"},
+	{MacroActionVariable::Type::FIND_AND_REPLACE,
+	 "AdvSceneSwitcher.action.variable.type.findAndReplace"},
+	{MacroActionVariable::Type::MATH_EXPRESSION,
+	 "AdvSceneSwitcher.action.variable.type.mathExpression"},
 };
 
 static void apppend(Variable &var, const std::string &value)
@@ -39,14 +48,14 @@ static void apppend(Variable &var, const std::string &value)
 
 static void modifyNumValue(Variable &var, double val, const bool increment)
 {
-	double current;
-	if (!var.DoubleValue(current)) {
+	auto current = var.DoubleValue();
+	if (!current.has_value()) {
 		return;
 	}
 	if (increment) {
-		var.SetValue(current + val);
+		var.SetValue(*current + val);
 	} else {
-		var.SetValue(current - val);
+		var.SetValue(*current - val);
 	}
 }
 
@@ -95,9 +104,26 @@ void MacroActionVariable::HandleRegexSubString(Variable *var)
 	var->SetValue(match.captured(0).toStdString());
 }
 
+void MacroActionVariable::HandleFindAndReplace(Variable *var)
+{
+	auto value = var->Value();
+	ReplaceAll(value, _findStr, _replaceStr);
+	var->SetValue(value);
+}
+
+void MacroActionVariable::HandleMathExpression(Variable *var)
+{
+	auto result = EvalMathExpression(_mathExpression);
+	if (std::holds_alternative<std::string>(result)) {
+		blog(LOG_WARNING, "%s", std::get<std::string>(result).c_str());
+		return;
+	}
+	var->SetValue(std::get<double>(result));
+}
+
 bool MacroActionVariable::PerformAction()
 {
-	auto var = GetVariableByName(_variableName);
+	auto var = _variable.lock();
 	if (!var) {
 		return true;
 	}
@@ -110,7 +136,7 @@ bool MacroActionVariable::PerformAction()
 		apppend(*var, _strValue);
 		break;
 	case Type::APPEND_VAR: {
-		auto var2 = GetVariableByName(_variable2Name);
+		auto var2 = _variable2.lock();
 		if (!var2) {
 			return true;
 		}
@@ -140,19 +166,27 @@ bool MacroActionVariable::PerformAction()
 		break;
 	}
 	case Type::ROUND_TO_INT: {
-		double curValue;
-		if (!var->DoubleValue(curValue)) {
+		auto curValue = var->DoubleValue();
+		if (!curValue.has_value()) {
 			return true;
 		}
-		var->SetValue(std::to_string(int(std::round(curValue))));
+		var->SetValue(std::to_string(int(std::round(*curValue))));
 		return true;
 	}
 	case Type::SUBSTRING: {
 		if (_regex.Enabled()) {
-			HandleRegexSubString(var);
+			HandleRegexSubString(var.get());
 			return true;
 		}
-		HandleIndexSubString(var);
+		HandleIndexSubString(var.get());
+		return true;
+	}
+	case Type::FIND_AND_REPLACE: {
+		HandleFindAndReplace(var.get());
+		return true;
+	}
+	case Type::MATH_EXPRESSION: {
+		HandleMathExpression(var.get());
 		return true;
 	}
 	}
@@ -163,8 +197,10 @@ bool MacroActionVariable::PerformAction()
 bool MacroActionVariable::Save(obs_data_t *obj) const
 {
 	MacroAction::Save(obj);
-	obs_data_set_string(obj, "variableName", _variableName.c_str());
-	obs_data_set_string(obj, "variable2Name", _variable2Name.c_str());
+	obs_data_set_string(obj, "variableName",
+			    GetWeakVariableName(_variable).c_str());
+	obs_data_set_string(obj, "variable2Name",
+			    GetWeakVariableName(_variable2).c_str());
 	obs_data_set_string(obj, "strValue", _strValue.c_str());
 	obs_data_set_double(obj, "numValue", _numValue);
 	obs_data_set_int(obj, "condition", static_cast<int>(_type));
@@ -173,15 +209,20 @@ bool MacroActionVariable::Save(obs_data_t *obj) const
 	obs_data_set_int(obj, "subStringSize", _subStringSize);
 	obs_data_set_string(obj, "regexPattern", _regexPattern.c_str());
 	obs_data_set_int(obj, "regexMatchIdx", _regexMatchIdx);
+	obs_data_set_string(obj, "findStr", _findStr.c_str());
+	obs_data_set_string(obj, "replaceStr", _replaceStr.c_str());
 	_regex.Save(obj);
+	_mathExpression.Save(obj, "mathExpression");
 	return true;
 }
 
 bool MacroActionVariable::Load(obs_data_t *obj)
 {
 	MacroAction::Load(obj);
-	_variableName = obs_data_get_string(obj, "variableName");
-	_variable2Name = obs_data_get_string(obj, "variable2Name");
+	_variable =
+		GetWeakVariableByName(obs_data_get_string(obj, "variableName"));
+	_variable2 = GetWeakVariableByName(
+		obs_data_get_string(obj, "variable2Name"));
 	_strValue = obs_data_get_string(obj, "strValue");
 	_numValue = obs_data_get_double(obj, "numValue");
 	_type = static_cast<Type>(obs_data_get_int(obj, "condition"));
@@ -191,6 +232,9 @@ bool MacroActionVariable::Load(obs_data_t *obj)
 	_regex.Load(obj);
 	_regexPattern = obs_data_get_string(obj, "regexPattern");
 	_regexMatchIdx = obs_data_get_int(obj, "regexMatchIdx");
+	_findStr = obs_data_get_string(obj, "findStr");
+	_replaceStr = obs_data_get_string(obj, "replaceStr");
+	_mathExpression.Load(obj, "mathExpression");
 	return true;
 }
 
@@ -202,7 +246,7 @@ bool MacroActionVariable::PostLoad()
 
 std::string MacroActionVariable::GetShortDesc() const
 {
-	return _variableName;
+	return GetWeakVariableName(_variable);
 }
 
 void MacroActionVariable::SetSegmentIndexValue(int value)
@@ -303,7 +347,12 @@ MacroActionVariableEdit::MacroActionVariableEdit(
 	  _subStringSize(new QSpinBox()),
 	  _regex(new RegexConfigWidget(parent)),
 	  _regexPattern(new ResizingPlainTextEdit(this, 10, 1, 1)),
-	  _regexMatchIdx(new QSpinBox())
+	  _regexMatchIdx(new QSpinBox()),
+	  _findReplaceLayout(new QHBoxLayout()),
+	  _findStr(new ResizingPlainTextEdit(this, 10, 1, 1)),
+	  _replaceStr(new ResizingPlainTextEdit(this, 10, 1, 1)),
+	  _mathExpression(new VariableLineEdit(this)),
+	  _mathExpressionResult(new QLabel())
 {
 	_numValue->setMinimum(-9999999999);
 	_numValue->setMaximum(9999999999);
@@ -348,6 +397,12 @@ MacroActionVariableEdit::MacroActionVariableEdit(
 			 SLOT(RegexPatternChanged()));
 	QWidget::connect(_regexMatchIdx, SIGNAL(valueChanged(int)), this,
 			 SLOT(RegexMatchIdxChanged(int)));
+	QWidget::connect(_findStr, SIGNAL(textChanged()), this,
+			 SLOT(FindStrValueChanged()));
+	QWidget::connect(_replaceStr, SIGNAL(textChanged()), this,
+			 SLOT(ReplaceStrValueChanged()));
+	QWidget::connect(_mathExpression, SIGNAL(editingFinished()), this,
+			 SLOT(MathExpressionChanged()));
 
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{variables}}", _variables},
@@ -359,21 +414,29 @@ MacroActionVariableEdit::MacroActionVariableEdit(
 		{"{{subStringStart}}", _subStringStart},
 		{"{{subStringSize}}", _subStringSize},
 		{"{{regexMatchIdx}}", _regexMatchIdx},
+		{"{{findStr}}", _findStr},
+		{"{{replaceStr}}", _replaceStr},
+		{"{{mathExpression}}", _mathExpression},
 	};
 	auto entryLayout = new QHBoxLayout;
-	placeWidgets(obs_module_text("AdvSceneSwitcher.action.variable.entry"),
+	PlaceWidgets(obs_module_text("AdvSceneSwitcher.action.variable.entry"),
 		     entryLayout, widgetPlaceholders);
 
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.action.variable.entry.substringIndex"),
 		_subStringIndexEntryLayout, widgetPlaceholders);
 
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.action.variable.entry.substringRegex"),
 		_subStringRegexEntryLayout, widgetPlaceholders);
 
+	PlaceWidgets(
+		obs_module_text(
+			"AdvSceneSwitcher.action.variable.entry.findAndReplace"),
+		_findReplaceLayout, widgetPlaceholders, false);
+
 	auto regexConfigLayout = new QHBoxLayout;
 	regexConfigLayout->addWidget(_regex);
 	regexConfigLayout->addStretch();
@@ -388,6 +451,8 @@ MacroActionVariableEdit::MacroActionVariableEdit(
 	layout->addLayout(_substringLayout);
 	layout->addWidget(_segmentValueStatus);
 	layout->addWidget(_segmentValue);
+	layout->addLayout(_findReplaceLayout);
+	layout->addWidget(_mathExpressionResult);
 	setLayout(layout);
 
 	_entryData = entryData;
@@ -406,8 +471,8 @@ void MacroActionVariableEdit::UpdateEntryData()
 		return;
 	}
 
-	_variables->SetVariable(_entryData->_variableName);
-	_variables2->SetVariable(_entryData->_variable2Name);
+	_variables->SetVariable(_entryData->_variable);
+	_variables2->SetVariable(_entryData->_variable2);
 	_actions->setCurrentIndex(static_cast<int>(_entryData->_type));
 	_strValue->setPlainText(QString::fromStdString(_entryData->_strValue));
 	_numValue->setValue(_entryData->_numValue);
@@ -418,6 +483,10 @@ void MacroActionVariableEdit::UpdateEntryData()
 	_regexPattern->setPlainText(
 		QString::fromStdString(_entryData->_regexPattern));
 	_regexMatchIdx->setValue(_entryData->_regexMatchIdx + 1);
+	_findStr->setPlainText(QString::fromStdString(_entryData->_findStr));
+	_replaceStr->setPlainText(
+		QString::fromStdString(_entryData->_replaceStr));
+	_mathExpression->setText(_entryData->_mathExpression);
 	SetWidgetVisibility();
 }
 
@@ -427,8 +496,8 @@ void MacroActionVariableEdit::VariableChanged(const QString &text)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_variableName = text.toStdString();
+	auto lock = LockContext();
+	_entryData->_variable = GetWeakVariableByQString(text);
 }
 
 void MacroActionVariableEdit::Variable2Changed(const QString &text)
@@ -437,8 +506,8 @@ void MacroActionVariableEdit::Variable2Changed(const QString &text)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_variable2Name = text.toStdString();
+	auto lock = LockContext();
+	_entryData->_variable2 = GetWeakVariableByQString(text);
 }
 
 void MacroActionVariableEdit::ActionChanged(int value)
@@ -447,7 +516,7 @@ void MacroActionVariableEdit::ActionChanged(int value)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_type = static_cast<MacroActionVariable::Type>(value);
 	SetWidgetVisibility();
 
@@ -464,9 +533,10 @@ void MacroActionVariableEdit::StrValueChanged()
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_strValue = _strValue->toPlainText().toStdString();
 	adjustSize();
+	updateGeometry();
 }
 
 void MacroActionVariableEdit::NumValueChanged(double val)
@@ -475,7 +545,7 @@ void MacroActionVariableEdit::NumValueChanged(double val)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_numValue = val;
 }
 
@@ -485,7 +555,7 @@ void MacroActionVariableEdit::SegmentIndexChanged(int val)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->SetSegmentIndexValue(val - 1);
 	MarkSelectedSegment();
 }
@@ -588,7 +658,7 @@ void MacroActionVariableEdit::SubStringStartChanged(int val)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_subStringStart = val - 1;
 }
 
@@ -598,7 +668,7 @@ void MacroActionVariableEdit::SubStringSizeChanged(int val)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_subStringSize = val;
 }
 
@@ -608,7 +678,7 @@ void MacroActionVariableEdit::RegexChanged(RegexConfig conf)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_regex = conf;
 
 	SetWidgetVisibility();
@@ -620,9 +690,10 @@ void MacroActionVariableEdit::RegexPatternChanged()
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_regexPattern = _regexPattern->toPlainText().toStdString();
 	adjustSize();
+	updateGeometry();
 }
 
 void MacroActionVariableEdit::RegexMatchIdxChanged(int val)
@@ -631,10 +702,56 @@ void MacroActionVariableEdit::RegexMatchIdxChanged(int val)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_regexMatchIdx = val - 1;
 }
 
+void MacroActionVariableEdit::FindStrValueChanged()
+{
+	if (_loading || !_entryData) {
+		return;
+	}
+
+	auto lock = LockContext();
+	_entryData->_findStr = _findStr->toPlainText().toStdString();
+	adjustSize();
+	updateGeometry();
+}
+
+void MacroActionVariableEdit::ReplaceStrValueChanged()
+{
+	if (_loading || !_entryData) {
+		return;
+	}
+
+	auto lock = LockContext();
+	_entryData->_replaceStr = _replaceStr->toPlainText().toStdString();
+	adjustSize();
+	updateGeometry();
+}
+
+void MacroActionVariableEdit::MathExpressionChanged()
+{
+	if (_loading || !_entryData) {
+		return;
+	}
+
+	auto lock = LockContext();
+	_entryData->_mathExpression = _mathExpression->text().toStdString();
+
+	// In case of invalid expression display an error
+	auto result = EvalMathExpression(_entryData->_mathExpression);
+	auto hasError = std::holds_alternative<std::string>(result);
+	if (hasError) {
+		_mathExpressionResult->setText(
+			QString::fromStdString(std::get<std::string>(result)));
+	}
+	_mathExpressionResult->setVisible(hasError);
+
+	adjustSize();
+	updateGeometry();
+}
+
 void MacroActionVariableEdit::MarkSelectedSegment()
 {
 	if (switcher->disableHints) {
@@ -701,16 +818,24 @@ void MacroActionVariableEdit::SetWidgetVisibility()
 			MacroActionVariable::Type::SET_ACTION_VALUE ||
 		_entryData->_type ==
 			MacroActionVariable::Type::SET_CONDITION_VALUE);
-	setLayoutVisible(_substringLayout,
+	SetLayoutVisible(_substringLayout,
 			 _entryData->_type ==
 				 MacroActionVariable::Type::SUBSTRING);
 	if (_entryData->_type == MacroActionVariable::Type::SUBSTRING) {
 		bool showRegex = _entryData->_regex.Enabled();
-		setLayoutVisible(_subStringIndexEntryLayout, !showRegex);
-		setLayoutVisible(_subStringRegexEntryLayout, showRegex);
+		SetLayoutVisible(_subStringIndexEntryLayout, !showRegex);
+		SetLayoutVisible(_subStringRegexEntryLayout, showRegex);
 		_regexPattern->setVisible(showRegex);
 	}
+	SetLayoutVisible(_findReplaceLayout,
+			 _entryData->_type ==
+				 MacroActionVariable::Type::FIND_AND_REPLACE);
+	_mathExpression->setVisible(_entryData->_type ==
+				    MacroActionVariable::Type::MATH_EXPRESSION);
+	_mathExpressionResult->hide();
 
 	adjustSize();
 	updateGeometry();
 }
+
+} // namespace advss
