@@ -1,8 +1,10 @@
 #pragma once
 #include "macro-action-edit.hpp"
-#include "resizing-text-edit.hpp"
 #include "regex-config.hpp"
-#include "variable.hpp"
+#include "resizing-text-edit.hpp"
+#include "variable-line-edit.hpp"
+
+namespace advss {
 
 class MacroActionVariable : public MacroAction {
 public:
@@ -31,11 +33,13 @@ public:
 		SET_ACTION_VALUE,
 		ROUND_TO_INT,
 		SUBSTRING,
+		FIND_AND_REPLACE,
+		MATH_EXPRESSION,
 	};
 
 	Type _type = Type::SET_FIXED_VALUE;
-	std::string _variableName = "";
-	std::string _variable2Name = "";
+	std::weak_ptr<Variable> _variable;
+	std::weak_ptr<Variable> _variable2;
 	std::string _strValue = "";
 	double _numValue = 0;
 	int _subStringStart = 0;
@@ -43,11 +47,19 @@ public:
 	RegexConfig _regex = RegexConfig::PartialMatchRegexConfig();
 	std::string _regexPattern = ".*";
 	int _regexMatchIdx = 0;
+	std::string _findStr = obs_module_text(
+		"AdvSceneSwitcher.action.variable.findAndReplace.find");
+	std::string _replaceStr = obs_module_text(
+		"AdvSceneSwitcher.action.variable.findAndReplace.replace");
+	StringVariable _mathExpression = obs_module_text(
+		"AdvSceneSwitcher.action.variable.mathExpression.example");
 
 private:
 	void DecrementCurrentSegmentVariableRef();
 	void HandleIndexSubString(Variable *);
 	void HandleRegexSubString(Variable *);
+	void HandleFindAndReplace(Variable *);
+	void HandleMathExpression(Variable *);
 
 	std::weak_ptr<MacroSegment> _macroSegment;
 	int _segmentIdxLoadValue = -1;
@@ -85,6 +97,9 @@ private slots:
 	void RegexChanged(RegexConfig conf);
 	void RegexPatternChanged();
 	void RegexMatchIdxChanged(int val);
+	void FindStrValueChanged();
+	void ReplaceStrValueChanged();
+	void MathExpressionChanged();
 
 signals:
 	void HeaderInfoChanged(const QString &);
@@ -106,6 +121,11 @@ protected:
 	RegexConfigWidget *_regex;
 	ResizingPlainTextEdit *_regexPattern;
 	QSpinBox *_regexMatchIdx;
+	QHBoxLayout *_findReplaceLayout;
+	ResizingPlainTextEdit *_findStr;
+	ResizingPlainTextEdit *_replaceStr;
+	VariableLineEdit *_mathExpression;
+	QLabel *_mathExpressionResult;
 	std::shared_ptr<MacroActionVariable> _entryData;
 
 private:
@@ -116,3 +136,5 @@ private:
 	QTimer _timer;
 	bool _loading = true;
 };
+
+} // namespace advss
