@@ -8,6 +8,8 @@
 #include <QListWidget>
 #include <QStringList>
 
+namespace advss {
+
 class MacroActionWebsocket : public MacroAction {
 public:
 	MacroActionWebsocket(Macro *m) : MacroAction(m) {}
@@ -28,9 +30,8 @@ public:
 	};
 
 	Type _type = Type::REQUEST;
-	VariableResolvingString _message =
-		obs_module_text("AdvSceneSwitcher.enterText");
-	std::string _connection;
+	StringVariable _message = obs_module_text("AdvSceneSwitcher.enterText");
+	std::weak_ptr<Connection> _connection;
 
 private:
 	void SendRequest();
@@ -75,3 +76,5 @@ private:
 	QHBoxLayout *_editLayout;
 	bool _loading = true;
 };
+
+} // namespace advss
