@@ -1,5 +1,6 @@
 #include "macro-action.hpp"
-#include "advanced-scene-switcher.hpp"
+
+namespace advss {
 
 MacroAction::MacroAction(Macro *m, bool supportsVariableValue)
 	: MacroSegment(m, supportsVariableValue)
@@ -29,9 +30,10 @@ MacroRefAction::MacroRefAction(Macro *m, bool supportsVariableValue)
 {
 }
 
-void MacroRefAction::ResolveMacroRef()
+bool MacroRefAction::PostLoad()
 {
-	_macro.UpdateRef();
+	_macro.PostLoad();
+	return true;
 }
 
 MultiMacroRefAction::MultiMacroRefAction(Macro *m, bool supportsVariableValue)
@@ -39,9 +41,12 @@ MultiMacroRefAction::MultiMacroRefAction(Macro *m, bool supportsVariableValue)
 {
 }
 
-void MultiMacroRefAction::ResolveMacroRef()
+bool MultiMacroRefAction::PostLoad()
 {
-	for (auto &m : _macros) {
-		m.UpdateRef();
+	for (auto &macro : _macros) {
+		macro.PostLoad();
 	}
+	return true;
 }
+
+} // namespace advss
