@@ -1,9 +1,12 @@
-#include "macro-condition-edit.hpp"
 #include "macro-condition-cursor.hpp"
+#include "switcher-data.hpp"
+#include "platform-funcs.hpp"
 #include "utility.hpp"
-#include "advanced-scene-switcher.hpp"
 
 #include <QScreen>
+#include <QGuiApplication>
+
+namespace advss {
 
 const std::string MacroConditionCursor::id = "cursor";
 
@@ -49,7 +52,7 @@ bool MacroConditionCursor::CheckClick()
 bool MacroConditionCursor::CheckCondition()
 {
 	bool ret = false;
-	std::pair<int, int> cursorPos = getCursorPos();
+	std::pair<int, int> cursorPos = GetCursorPos();
 	switch (_condition) {
 	case Condition::REGION:
 		ret = cursorPos.first >= _minX && cursorPos.second >= _minY &&
@@ -77,10 +80,11 @@ bool MacroConditionCursor::Save(obs_data_t *obj) const
 	MacroCondition::Save(obj);
 	obs_data_set_int(obj, "condition", static_cast<int>(_condition));
 	obs_data_set_int(obj, "button", static_cast<int>(_button));
-	obs_data_set_int(obj, "minX", _minX);
-	obs_data_set_int(obj, "minY", _minY);
-	obs_data_set_int(obj, "maxX", _maxX);
-	obs_data_set_int(obj, "maxY", _maxY);
+	_minX.Save(obj, "minX");
+	_minY.Save(obj, "minY");
+	_maxX.Save(obj, "maxX");
+	_maxY.Save(obj, "maxY");
+	obs_data_set_int(obj, "version", 1);
 	return true;
 }
 
@@ -89,10 +93,17 @@ bool MacroConditionCursor::Load(obs_data_t *obj)
 	MacroCondition::Load(obj);
 	_condition = static_cast<Condition>(obs_data_get_int(obj, "condition"));
 	_button = static_cast<Button>(obs_data_get_int(obj, "button"));
-	_minX = obs_data_get_int(obj, "minX");
-	_minY = obs_data_get_int(obj, "minY");
-	_maxX = obs_data_get_int(obj, "maxX");
-	_maxY = obs_data_get_int(obj, "maxY");
+	if (!obs_data_has_user_value(obj, "version")) {
+		_minX = obs_data_get_int(obj, "minX");
+		_minY = obs_data_get_int(obj, "minY");
+		_maxX = obs_data_get_int(obj, "maxX");
+		_maxY = obs_data_get_int(obj, "maxY");
+	} else {
+		_minX.Load(obj, "minX");
+		_minY.Load(obj, "minY");
+		_maxX.Load(obj, "maxX");
+		_maxY.Load(obj, "maxY");
+	}
 	return true;
 }
 
@@ -115,10 +126,10 @@ static inline void populateButtonSelection(QComboBox *list)
 MacroConditionCursorEdit::MacroConditionCursorEdit(
 	QWidget *parent, std::shared_ptr<MacroConditionCursor> entryData)
 	: QWidget(parent),
-	  _minX(new QSpinBox()),
-	  _minY(new QSpinBox()),
-	  _maxX(new QSpinBox()),
-	  _maxY(new QSpinBox()),
+	  _minX(new VariableSpinBox()),
+	  _minY(new VariableSpinBox()),
+	  _maxX(new VariableSpinBox()),
+	  _maxY(new VariableSpinBox()),
 	  _conditions(new QComboBox()),
 	  _buttons(new QComboBox()),
 	  _frameToggle(new QPushButton(obs_module_text(
@@ -151,14 +162,22 @@ MacroConditionCursorEdit::MacroConditionCursorEdit(
 			 SLOT(ConditionChanged(int)));
 	QWidget::connect(_buttons, SIGNAL(currentIndexChanged(int)), this,
 			 SLOT(ButtonChanged(int)));
-	QWidget::connect(_minX, SIGNAL(valueChanged(int)), this,
-			 SLOT(MinXChanged(int)));
-	QWidget::connect(_minY, SIGNAL(valueChanged(int)), this,
-			 SLOT(MinYChanged(int)));
-	QWidget::connect(_maxX, SIGNAL(valueChanged(int)), this,
-			 SLOT(MaxXChanged(int)));
-	QWidget::connect(_maxY, SIGNAL(valueChanged(int)), this,
-			 SLOT(MaxYChanged(int)));
+	QWidget::connect(
+		_minX,
+		SIGNAL(NumberVariableChanged(const NumberVariable<int> &)),
+		this, SLOT(MinXChanged(const NumberVariable<int> &)));
+	QWidget::connect(
+		_minY,
+		SIGNAL(NumberVariableChanged(const NumberVariable<int> &)),
+		this, SLOT(MinYChanged(const NumberVariable<int> &)));
+	QWidget::connect(
+		_maxX,
+		SIGNAL(NumberVariableChanged(const NumberVariable<int> &)),
+		this, SLOT(MaxXChanged(const NumberVariable<int> &)));
+	QWidget::connect(
+		_maxY,
+		SIGNAL(NumberVariableChanged(const NumberVariable<int> &)),
+		this, SLOT(MaxYChanged(const NumberVariable<int> &)));
 	QWidget::connect(_frameToggle, SIGNAL(clicked()), this,
 			 SLOT(ToggleFrame()));
 
@@ -174,10 +193,10 @@ MacroConditionCursorEdit::MacroConditionCursorEdit(
 		{"{{toggleFrameButton}}", _frameToggle},
 	};
 	QHBoxLayout *line1 = new QHBoxLayout;
-	placeWidgets(obs_module_text(
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.condition.cursor.entry.line1"),
 		     line1, widgetPlaceholders);
-	placeWidgets(obs_module_text(
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.condition.cursor.entry.line2"),
 		     _curentPosLayout, widgetPlaceholders);
 	QVBoxLayout *mainLayout = new QVBoxLayout;
@@ -200,7 +219,7 @@ void MacroConditionCursorEdit::ConditionChanged(int index)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_condition = static_cast<MacroConditionCursor::Condition>(
 		_conditions->itemData(index).toInt());
 	SetWidgetVisibility();
@@ -212,58 +231,58 @@ void MacroConditionCursorEdit::ButtonChanged(int index)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_button = static_cast<MacroConditionCursor::Button>(
 		_buttons->itemData(index).toInt());
 }
 
-void MacroConditionCursorEdit::MinXChanged(int pos)
+void MacroConditionCursorEdit::MinXChanged(const NumberVariable<int> &pos)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_minX = pos;
 	SetupFrame();
 }
 
-void MacroConditionCursorEdit::MinYChanged(int pos)
+void MacroConditionCursorEdit::MinYChanged(const NumberVariable<int> &pos)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_minY = pos;
 	SetupFrame();
 }
 
-void MacroConditionCursorEdit::MaxXChanged(int pos)
+void MacroConditionCursorEdit::MaxXChanged(const NumberVariable<int> &pos)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_maxX = pos;
 	SetupFrame();
 }
 
-void MacroConditionCursorEdit::MaxYChanged(int pos)
+void MacroConditionCursorEdit::MaxYChanged(const NumberVariable<int> &pos)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_maxY = pos;
 	SetupFrame();
 }
 
 void MacroConditionCursorEdit::UpdateCursorPos()
 {
-	std::pair<int, int> position = getCursorPos();
+	std::pair<int, int> position = GetCursorPos();
 	_xPos->setText(QString::number(position.first));
 	_yPos->setText(QString::number(position.second));
 }
@@ -291,7 +310,7 @@ void MacroConditionCursorEdit::SetWidgetVisibility()
 	_maxX->setVisible(isRegionCondition);
 	_maxY->setVisible(isRegionCondition);
 	_frameToggle->setVisible(isRegionCondition);
-	setLayoutVisible(_curentPosLayout, isRegionCondition);
+	SetLayoutVisible(_curentPosLayout, isRegionCondition);
 	if (_frame.isVisible()) {
 		ToggleFrame();
 	}
@@ -334,18 +353,20 @@ void MacroConditionCursorEdit::SetupFrame()
 			      Qt::WindowStaysOnTopHint);
 	_frame.setAttribute(Qt::WA_TranslucentBackground, true);
 
-	if (_entryData) {
-		QRect rect(_entryData->_minX, _entryData->_minY,
-			   _entryData->_maxX - _entryData->_minX,
-			   _entryData->_maxY - _entryData->_minY);
-
-		if (rect.size().height() == 0 || rect.size().width() == 0) {
-			_frame.setGeometry(0, 0, 1, 1);
-			return;
-		}
-
-		_frame.setGeometry(getScreenUnionRect().intersected(rect));
+	if (!_entryData) {
+		return;
 	}
+
+	QRect rect(_entryData->_minX, _entryData->_minY,
+		   _entryData->_maxX - _entryData->_minX,
+		   _entryData->_maxY - _entryData->_minY);
+
+	if (rect.size().height() <= 0 || rect.size().width() <= 0) {
+		_frame.setGeometry(0, 0, 1, 1);
+		return;
+	}
+
+	_frame.setGeometry(getScreenUnionRect().intersected(rect));
 }
 
 void MacroConditionCursorEdit::UpdateEntryData()
@@ -356,9 +377,11 @@ void MacroConditionCursorEdit::UpdateEntryData()
 
 	_conditions->setCurrentIndex(static_cast<int>(_entryData->_condition));
 	_buttons->setCurrentIndex(static_cast<int>(_entryData->_button));
-	_minX->setValue(_entryData->_minX);
-	_minY->setValue(_entryData->_minY);
-	_maxX->setValue(_entryData->_maxX);
-	_maxY->setValue(_entryData->_maxY);
+	_minX->SetValue(_entryData->_minX);
+	_minY->SetValue(_entryData->_minY);
+	_maxX->SetValue(_entryData->_maxX);
+	_maxY->SetValue(_entryData->_maxY);
 	SetWidgetVisibility();
 }
+
+} // namespace advss
