@@ -1,9 +1,12 @@
 #include "advanced-scene-switcher.hpp"
+#include "switcher-data.hpp"
 #include "macro-condition-edit.hpp"
 #include "macro-condition-scene.hpp"
 #include "section.hpp"
 #include "utility.hpp"
 
+namespace advss {
+
 std::map<std::string, MacroConditionInfo> &MacroConditionFactory::GetMap()
 {
 	static std::map<std::string, MacroConditionInfo> _methods;
@@ -121,15 +124,14 @@ DurationModifierEdit::DurationModifierEdit(QWidget *parent)
 	_duration = new DurationSelection(parent);
 	_toggle = new QPushButton(parent);
 	_toggle->setMaximumWidth(22);
-	_toggle->setIcon(
-		QIcon(QString::fromStdString(getDataFilePath("res/time.svg"))));
+	const auto path = QString::fromStdString(GetDataFilePath(
+		"res/images/" + GetThemeTypeName() + "Time.svg"));
+	_toggle->setIcon(QIcon(path));
 	populateDurationModifierTypes(_condition);
 	QWidget::connect(_condition, SIGNAL(currentIndexChanged(int)), this,
 			 SLOT(_ModifierChanged(int)));
 	QObject::connect(_duration, &DurationSelection::DurationChanged, this,
 			 &DurationModifierEdit::DurationChanged);
-	QObject::connect(_duration, &DurationSelection::UnitChanged, this,
-			 &DurationModifierEdit::UnitChanged);
 	QWidget::connect(_toggle, SIGNAL(clicked()), this,
 			 SLOT(ToggleClicked()));
 
@@ -150,11 +152,6 @@ void DurationModifierEdit::SetValue(DurationModifier &value)
 	_duration->setVisible(value.GetType() != DurationModifier::Type::NONE);
 }
 
-void DurationModifierEdit::SetUnit(DurationUnit u)
-{
-	_duration->SetUnit(u);
-}
-
 void DurationModifierEdit::SetDuration(const Duration &d)
 {
 	_duration->SetDuration(d);
@@ -195,10 +192,8 @@ MacroConditionEdit::MacroConditionEdit(
 	QWidget::connect(_conditionSelection,
 			 SIGNAL(currentTextChanged(const QString &)), this,
 			 SLOT(ConditionSelectionChanged(const QString &)));
-	QWidget::connect(_dur, SIGNAL(DurationChanged(double)), this,
-			 SLOT(DurationChanged(double)));
-	QWidget::connect(_dur, SIGNAL(UnitChanged(DurationUnit)), this,
-			 SLOT(DurationUnitChanged(DurationUnit)));
+	QWidget::connect(_dur, SIGNAL(DurationChanged(const Duration &)), this,
+			 SLOT(DurationChanged(const Duration &)));
 	QWidget::connect(_dur, SIGNAL(ModifierChanged(DurationModifier::Type)),
 			 this,
 			 SLOT(DurationModifierChanged(DurationModifier::Type)));
@@ -241,7 +236,7 @@ void MacroConditionEdit::LogicSelectionChanged(int idx)
 		type = static_cast<LogicType>(idx + logic_root_offset);
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	(*_entryData)->SetLogicType(type);
 }
 
@@ -308,7 +303,7 @@ void MacroConditionEdit::ConditionSelectionChanged(const QString &text)
 	_dur->SetValue(temp);
 	HeaderInfoChanged("");
 	{
-		std::lock_guard<std::mutex> lock(switcher->m);
+		auto lock = LockContext();
 		auto logic = (*_entryData)->GetLogicType();
 		_entryData->reset();
 		*_entryData = MacroConditionFactory::Create(id, macro);
@@ -324,13 +319,13 @@ void MacroConditionEdit::ConditionSelectionChanged(const QString &text)
 	SetFocusPolicyOfWidgets();
 }
 
-void MacroConditionEdit::DurationChanged(double seconds)
+void MacroConditionEdit::DurationChanged(const Duration &seconds)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	(*_entryData)->SetDuration(seconds);
 }
 
@@ -340,20 +335,10 @@ void MacroConditionEdit::DurationModifierChanged(DurationModifier::Type m)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	(*_entryData)->SetDurationModifier(m);
 }
 
-void MacroConditionEdit::DurationUnitChanged(DurationUnit unit)
-{
-	if (_loading || !_entryData) {
-		return;
-	}
-
-	std::lock_guard<std::mutex> lock(switcher->m);
-	(*_entryData)->SetDurationUnit(unit);
-}
-
 std::shared_ptr<MacroSegment> MacroConditionEdit::Data()
 {
 	return *_entryData;
@@ -361,7 +346,7 @@ std::shared_ptr<MacroSegment> MacroConditionEdit::Data()
 
 void AdvSceneSwitcher::AddMacroCondition(int idx)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -375,7 +360,7 @@ void AdvSceneSwitcher::AddMacroCondition(int idx)
 	if (idx >= 1) {
 		id = macro->Conditions().at(idx - 1)->GetId();
 		if (idx == 1) {
-			logic = LogicType::NONE;
+			logic = LogicType::OR;
 		} else {
 			logic = macro->Conditions().at(idx - 1)->GetLogicType();
 		}
@@ -385,7 +370,7 @@ void AdvSceneSwitcher::AddMacroCondition(int idx)
 		logic = LogicType::ROOT_NONE;
 	}
 	{
-		std::lock_guard<std::mutex> lock(switcher->m);
+		auto lock = LockContext();
 		auto cond = macro->Conditions().emplace(
 			macro->Conditions().begin() + idx,
 			MacroConditionFactory::Create(id, macro.get()));
@@ -409,7 +394,7 @@ void AdvSceneSwitcher::AddMacroCondition(int idx)
 
 void AdvSceneSwitcher::on_conditionAdd_clicked()
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -427,7 +412,7 @@ void AdvSceneSwitcher::on_conditionAdd_clicked()
 
 void AdvSceneSwitcher::RemoveMacroCondition(int idx)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -437,7 +422,7 @@ void AdvSceneSwitcher::RemoveMacroCondition(int idx)
 	}
 
 	{
-		std::lock_guard<std::mutex> lock(switcher->m);
+		auto lock = LockContext();
 		conditionsList->Remove(idx);
 		macro->Conditions().erase(macro->Conditions().begin() + idx);
 		macro->UpdateConditionIndices();
@@ -458,7 +443,7 @@ void AdvSceneSwitcher::RemoveMacroCondition(int idx)
 void AdvSceneSwitcher::on_conditionRemove_clicked()
 {
 	if (currentConditionIdx == -1) {
-		auto macro = getSelectedMacro();
+		auto macro = GetSelectedMacro();
 		if (!macro) {
 			return;
 		}
@@ -497,7 +482,7 @@ void AdvSceneSwitcher::SwapConditions(Macro *m, int pos1, int pos2)
 	}
 
 	bool root = pos1 == 0;
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	iter_swap(m->Conditions().begin() + pos1,
 		  m->Conditions().begin() + pos2);
 	m->UpdateConditionIndices();
@@ -525,7 +510,7 @@ void AdvSceneSwitcher::SwapConditions(Macro *m, int pos1, int pos2)
 
 void AdvSceneSwitcher::MoveMacroConditionUp(int idx)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -540,7 +525,7 @@ void AdvSceneSwitcher::MoveMacroConditionUp(int idx)
 
 void AdvSceneSwitcher::MoveMacroConditionDown(int idx)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -555,7 +540,7 @@ void AdvSceneSwitcher::MoveMacroConditionDown(int idx)
 
 void AdvSceneSwitcher::MacroConditionSelectionChanged(int idx)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -575,7 +560,7 @@ void AdvSceneSwitcher::MacroConditionSelectionChanged(int idx)
 
 void AdvSceneSwitcher::MacroConditionReorder(int to, int from)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -585,7 +570,7 @@ void AdvSceneSwitcher::MacroConditionReorder(int to, int from)
 		return;
 	}
 	{
-		std::lock_guard<std::mutex> lock(switcher->m);
+		auto lock = LockContext();
 		auto condition = macro->Conditions().at(from);
 		if (to == 0) {
 			condition->SetLogicType(LogicType::ROOT_NONE);
@@ -619,3 +604,5 @@ void AdvSceneSwitcher::MacroConditionReorder(int to, int from)
 	HighlightCondition(to);
 	emit(MacroSegmentOrderChanged());
 }
+
+} // namespace advss
