@@ -1,10 +1,10 @@
-#include "macro-condition-edit.hpp"
 #include "macro-condition-filter.hpp"
 #include "utility.hpp"
-#include "advanced-scene-switcher.hpp"
 
 #include <regex>
 
+namespace advss {
+
 const std::string MacroConditionFilter::id = "filter";
 
 bool MacroConditionFilter::_registered = MacroConditionFactory::Register(
@@ -22,47 +22,34 @@ static std::map<MacroConditionFilter::Condition, std::string>
 		 "AdvSceneSwitcher.condition.filter.type.settings"},
 };
 
-void MacroConditionFilter::ResolveVariables()
-{
-	if (_source.GetType() == SourceSelection::Type::SOURCE) {
-		return;
-	}
-
-	std::string name = GetWeakSourceName(_filter);
-	if (!name.empty()) {
-		_filterName = name;
-	}
-	_filter = GetWeakFilterByName(_source.GetSource(), _filterName.c_str());
-}
-
 bool MacroConditionFilter::CheckCondition()
 {
-	ResolveVariables();
-	if (!_source.GetSource()) {
+	auto filterWeakSource = _filter.GetFilter(_source);
+	if (!filterWeakSource) {
 		return false;
 	}
+	auto filterSource = obs_weak_source_get_source(filterWeakSource);
 
 	bool ret = false;
-	auto s = obs_weak_source_get_source(_filter);
-
 	switch (_condition) {
 	case Condition::ENABLED:
-		ret = obs_source_enabled(s);
+		ret = obs_source_enabled(filterSource);
 		break;
 	case Condition::DISABLED:
-		ret = !obs_source_enabled(s);
+		ret = !obs_source_enabled(filterSource);
 		break;
 	case Condition::SETTINGS:
-		ret = compareSourceSettings(_filter, _settings, _regex);
+		ret = CompareSourceSettings(filterWeakSource, _settings,
+					    _regex);
 		if (IsReferencedInVars()) {
-			SetVariableValue(getSourceSettings(_filter));
+			SetVariableValue(GetSourceSettings(filterWeakSource));
 		}
 		break;
 	default:
 		break;
 	}
 
-	obs_source_release(s);
+	obs_source_release(filterSource);
 
 	if (GetVariableValue().empty()) {
 		SetVariableValue(ret ? "true" : "false");
@@ -75,7 +62,7 @@ bool MacroConditionFilter::Save(obs_data_t *obj) const
 {
 	MacroCondition::Save(obj);
 	_source.Save(obj);
-	obs_data_set_string(obj, "filter", _filterName.c_str());
+	_filter.Save(obj, "filter");
 	obs_data_set_int(obj, "condition", static_cast<int>(_condition));
 	_settings.Save(obj, "settings");
 	_regex.Save(obj);
@@ -86,9 +73,7 @@ bool MacroConditionFilter::Load(obs_data_t *obj)
 {
 	MacroCondition::Load(obj);
 	_source.Load(obj);
-	_filterName = obs_data_get_string(obj, "filter");
-	_filter = GetWeakFilterByQString(_source.GetSource(),
-					 _filterName.c_str());
+	_filter.Load(obj, _source, "filter");
 	_condition = static_cast<Condition>(obs_data_get_int(obj, "condition"));
 	_settings.Load(obj, "settings");
 	_regex.Load(obj);
@@ -102,8 +87,8 @@ bool MacroConditionFilter::Load(obs_data_t *obj)
 
 std::string MacroConditionFilter::GetShortDesc() const
 {
-	if (_filter && !_source.ToString().empty()) {
-		return _source.ToString() + " - " + GetWeakSourceName(_filter);
+	if (!_filter.ToString().empty() && !_source.ToString().empty()) {
+		return _source.ToString() + " - " + _filter.ToString();
 	}
 	return "";
 }
@@ -119,14 +104,13 @@ MacroConditionFilterEdit::MacroConditionFilterEdit(
 	QWidget *parent, std::shared_ptr<MacroConditionFilter> entryData)
 	: QWidget(parent),
 	  _sources(new SourceSelectionWidget(this, QStringList(), true)),
-	  _filters(new QComboBox()),
+	  _filters(new FilterSelectionWidget(this, _sources, true)),
 	  _conditions(new QComboBox()),
 	  _getSettings(new QPushButton(obs_module_text(
 		  "AdvSceneSwitcher.condition.filter.getSettings"))),
 	  _settings(new VariableTextEdit(this)),
 	  _regex(new RegexConfigWidget(parent))
 {
-	_filters->setSizeAdjustPolicy(QComboBox::AdjustToContents);
 	populateConditionSelection(_conditions);
 	auto sources = GetSourcesWithFilterNames();
 	sources.sort();
@@ -135,8 +119,9 @@ MacroConditionFilterEdit::MacroConditionFilterEdit(
 	QWidget::connect(_sources,
 			 SIGNAL(SourceChanged(const SourceSelection &)), this,
 			 SLOT(SourceChanged(const SourceSelection &)));
-	QWidget::connect(_filters, SIGNAL(currentTextChanged(const QString &)),
-			 this, SLOT(FilterChanged(const QString &)));
+	QWidget::connect(_filters,
+			 SIGNAL(FilterChanged(const FilterSelection &)), this,
+			 SLOT(FilterChanged(const FilterSelection &)));
 	QWidget::connect(_conditions, SIGNAL(currentIndexChanged(int)), this,
 			 SLOT(ConditionChanged(int)));
 	QWidget::connect(_getSettings, SIGNAL(clicked()), this,
@@ -146,21 +131,24 @@ MacroConditionFilterEdit::MacroConditionFilterEdit(
 	QWidget::connect(_regex, SIGNAL(RegexConfigChanged(RegexConfig)), this,
 			 SLOT(RegexChanged(RegexConfig)));
 
-	QHBoxLayout *line1Layout = new QHBoxLayout;
-	QHBoxLayout *line2Layout = new QHBoxLayout;
-	QHBoxLayout *line3Layout = new QHBoxLayout;
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{sources}}", _sources},         {"{{filters}}", _filters},
 		{"{{conditions}}", _conditions},   {"{{settings}}", _settings},
 		{"{{getSettings}}", _getSettings}, {"{{regex}}", _regex},
 	};
-	placeWidgets(obs_module_text(
+	auto line1Layout = new QHBoxLayout;
+	line1Layout->setContentsMargins(0, 0, 0, 0);
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.condition.filter.entry.line1"),
 		     line1Layout, widgetPlaceholders);
-	placeWidgets(obs_module_text(
+	auto line2Layout = new QHBoxLayout;
+	line2Layout->setContentsMargins(0, 0, 0, 0);
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.condition.filter.entry.line2"),
 		     line2Layout, widgetPlaceholders, false);
-	placeWidgets(obs_module_text(
+	auto line3Layout = new QHBoxLayout;
+	line3Layout->setContentsMargins(0, 0, 0, 0);
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.condition.filter.entry.line3"),
 		     line3Layout, widgetPlaceholders);
 
@@ -180,25 +168,19 @@ void MacroConditionFilterEdit::SourceChanged(const SourceSelection &source)
 	if (_loading || !_entryData) {
 		return;
 	}
-	{
-		std::lock_guard<std::mutex> lock(switcher->m);
-		_entryData->_source = source;
-	}
-	_filters->clear();
-	populateFilterSelection(_filters, _entryData->_source.GetSource());
-	_filters->adjustSize();
+
+	auto lock = LockContext();
+	_entryData->_source = source;
 }
 
-void MacroConditionFilterEdit::FilterChanged(const QString &text)
+void MacroConditionFilterEdit::FilterChanged(const FilterSelection &filter)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_filterName = text.toStdString();
-	_entryData->_filter =
-		GetWeakFilterByQString(_entryData->_source.GetSource(), text);
+	auto lock = LockContext();
+	_entryData->_filter = filter;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
 }
@@ -209,7 +191,7 @@ void MacroConditionFilterEdit::ConditionChanged(int index)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_condition =
 		static_cast<MacroConditionFilter::Condition>(index);
 	SetSettingsSelectionVisible(_entryData->_condition ==
@@ -218,13 +200,15 @@ void MacroConditionFilterEdit::ConditionChanged(int index)
 
 void MacroConditionFilterEdit::GetSettingsClicked()
 {
-	if (_loading || !_entryData || !_entryData->_source.GetSource()) {
+	if (_loading || !_entryData ||
+	    !_entryData->_filter.GetFilter(_entryData->_source)) {
 		return;
 	}
 
-	QString json = formatJsonString(getSourceSettings(_entryData->_filter));
+	QString json = FormatJsonString(GetSourceSettings(
+		_entryData->_filter.GetFilter(_entryData->_source)));
 	if (_entryData->_regex.Enabled()) {
-		json = escapeForRegex(json);
+		json = EscapeForRegex(json);
 	}
 	_settings->setPlainText(json);
 }
@@ -235,7 +219,7 @@ void MacroConditionFilterEdit::SettingsChanged()
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_settings = _settings->toPlainText().toStdString();
 
 	adjustSize();
@@ -248,7 +232,7 @@ void MacroConditionFilterEdit::RegexChanged(RegexConfig conf)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_regex = conf;
 
 	adjustSize();
@@ -261,6 +245,7 @@ void MacroConditionFilterEdit::SetSettingsSelectionVisible(bool visible)
 	_getSettings->setVisible(visible);
 	_regex->setVisible(visible);
 	adjustSize();
+	updateGeometry();
 }
 
 void MacroConditionFilterEdit::UpdateEntryData()
@@ -270,9 +255,7 @@ void MacroConditionFilterEdit::UpdateEntryData()
 	}
 
 	_sources->SetSource(_entryData->_source);
-	populateFilterSelection(_filters, _entryData->_source.GetSource());
-	_filters->setCurrentText(
-		GetWeakSourceName(_entryData->_filter).c_str());
+	_filters->SetFilter(_entryData->_source, _entryData->_filter);
 	_conditions->setCurrentIndex(static_cast<int>(_entryData->_condition));
 	_settings->setPlainText(_entryData->_settings);
 	_regex->SetRegexConfig(_entryData->_regex);
@@ -282,3 +265,5 @@ void MacroConditionFilterEdit::UpdateEntryData()
 	adjustSize();
 	updateGeometry();
 }
+
+} // namespace advss
