@@ -1,7 +1,8 @@
-#include "macro-condition-edit.hpp"
 #include "macro-condition-macro.hpp"
+#include "macro.hpp"
 #include "utility.hpp"
-#include "advanced-scene-switcher.hpp"
+
+namespace advss {
 
 const std::string MacroConditionMacro::id = "macro";
 
@@ -44,11 +45,12 @@ bool MacroConditionMacro::CheckStateCondition()
 	// Note:
 	// Depending on the order the macro conditions are checked Matched() might
 	// still return the state of the previous interval
-	if (!_macro.get()) {
+	auto macro = _macro.GetMacro();
+	if (!macro) {
 		return false;
 	}
 
-	return _macro->Matched();
+	return macro->Matched();
 }
 
 bool MacroConditionMacro::CheckMultiStateCondition()
@@ -57,8 +59,9 @@ bool MacroConditionMacro::CheckMultiStateCondition()
 	// Depending on the order the macro conditions are checked Matched() might
 	// still return the state of the previous interval
 	int matchedCount = 0;
-	for (const auto &macro : _macros) {
-		if (!macro.get()) {
+	for (const auto &m : _macros) {
+		auto macro = m.GetMacro();
+		if (!macro) {
 			continue;
 		}
 		if (macro->Matched()) {
@@ -82,17 +85,18 @@ bool MacroConditionMacro::CheckMultiStateCondition()
 
 bool MacroConditionMacro::CheckCountCondition()
 {
-	if (!_macro.get()) {
+	auto macro = _macro.GetMacro();
+	if (!macro) {
 		return false;
 	}
 
 	switch (_counterCondition) {
 	case CounterCondition::BELOW:
-		return _macro->RunCount() < _count;
+		return macro->RunCount() < _count;
 	case CounterCondition::ABOVE:
-		return _macro->RunCount() > _count;
+		return macro->RunCount() > _count;
 	case CounterCondition::EQUAL:
-		return _macro->RunCount() == _count;
+		return macro->RunCount() == _count;
 	default:
 		break;
 	}
@@ -123,10 +127,11 @@ bool MacroConditionMacro::Save(obs_data_t *obj) const
 	_macro.Save(obj);
 	obs_data_set_int(obj, "type", static_cast<int>(_type));
 	obs_data_set_int(obj, "condition", static_cast<int>(_counterCondition));
-	obs_data_set_int(obj, "count", _count);
-	obs_data_set_int(obj, "multiStateCount", _multiSateCount);
+	_count.Save(obj, "count");
+	_multiSateCount.Save(obj, "multiStateCount");
 	obs_data_set_int(obj, "multiStateCondition",
 			 static_cast<int>(_multiSateCondition));
+	obs_data_set_int(obj, "version", 1);
 	return true;
 }
 
@@ -138,8 +143,6 @@ bool MacroConditionMacro::Load(obs_data_t *obj)
 	_type = static_cast<Type>(obs_data_get_int(obj, "type"));
 	_counterCondition = static_cast<CounterCondition>(
 		obs_data_get_int(obj, "condition"));
-	_count = obs_data_get_int(obj, "count");
-	_multiSateCount = obs_data_get_int(obj, "multiStateCount");
 	// TODO: Remove this fallback in future version
 	if (!obs_data_has_user_value(obj, "multiStateCondition")) {
 		_multiSateCondition = MultiStateCondition::ABOVE;
@@ -147,15 +150,27 @@ bool MacroConditionMacro::Load(obs_data_t *obj)
 		_multiSateCondition = static_cast<MultiStateCondition>(
 			obs_data_get_int(obj, "multiStateCondition"));
 	}
+
+	if (!obs_data_has_user_value(obj, "version")) {
+		_count = obs_data_get_int(obj, "count");
+		_multiSateCount = obs_data_get_int(obj, "multiStateCount");
+	} else {
+		_count.Load(obj, "count");
+		_multiSateCount.Load(obj, "multiStateCount");
+	}
+
 	return true;
 }
 
+bool MacroConditionMacro::PostLoad()
+{
+	return MacroRefCondition::PostLoad() &&
+	       MultiMacroRefCondtition::PostLoad();
+}
+
 std::string MacroConditionMacro::GetShortDesc() const
 {
-	if (_macro.get()) {
-		return _macro->Name();
-	}
-	return "";
+	return _macro.Name();
 }
 
 static inline void populateTypeSelection(QComboBox *list)
@@ -183,19 +198,19 @@ MacroConditionMacroEdit::MacroConditionMacroEdit(
 	QWidget *parent, std::shared_ptr<MacroConditionMacro> entryData)
 	: QWidget(parent),
 	  _macros(new MacroSelection(parent)),
-	  _types(new QComboBox(parent)),
-	  _counterConditions(new QComboBox(parent)),
-	  _count(new QSpinBox(parent)),
-	  _currentCount(new QLabel(parent)),
+	  _types(new QComboBox()),
+	  _counterConditions(new QComboBox()),
+	  _count(new VariableSpinBox()),
+	  _currentCount(new QLabel()),
 	  _pausedWarning(new QLabel(obs_module_text(
 		  "AdvSceneSwitcher.condition.macro.pausedWarning"))),
 	  _resetCount(new QPushButton(obs_module_text(
 		  "AdvSceneSwitcher.condition.macro.count.reset"))),
-	  _settingsLine1(new QHBoxLayout),
-	  _settingsLine2(new QHBoxLayout),
+	  _settingsLine1(new QHBoxLayout()),
+	  _settingsLine2(new QHBoxLayout()),
 	  _macroList(new MacroList(this, false, false)),
-	  _multiStateConditions(new QComboBox(parent)),
-	  _multiStateCount(new QSpinBox(parent))
+	  _multiStateConditions(new QComboBox()),
+	  _multiStateCount(new VariableSpinBox())
 {
 	_count->setMaximum(10000000);
 	populateTypeSelection(_types);
@@ -210,8 +225,10 @@ MacroConditionMacroEdit::MacroConditionMacroEdit(
 			 SLOT(TypeChanged(int)));
 	QWidget::connect(_counterConditions, SIGNAL(currentIndexChanged(int)),
 			 this, SLOT(CountConditionChanged(int)));
-	QWidget::connect(_count, SIGNAL(valueChanged(int)), this,
-			 SLOT(CountChanged(int)));
+	QWidget::connect(
+		_count,
+		SIGNAL(NumberVariableChanged(const NumberVariable<int> &)),
+		this, SLOT(CountChanged(const NumberVariable<int> &)));
 	QWidget::connect(_resetCount, SIGNAL(clicked()), this,
 			 SLOT(ResetClicked()));
 	QWidget::connect(_macroList, SIGNAL(Added(const std::string &)), this,
@@ -223,18 +240,21 @@ MacroConditionMacroEdit::MacroConditionMacroEdit(
 	QWidget::connect(_multiStateConditions,
 			 SIGNAL(currentIndexChanged(int)), this,
 			 SLOT(MultiStateConditionChanged(int)));
-	QWidget::connect(_multiStateCount, SIGNAL(valueChanged(int)), this,
-			 SLOT(MultiStateCountChanged(int)));
+	QWidget::connect(
+		_multiStateCount,
+		SIGNAL(NumberVariableChanged(const NumberVariable<int> &)),
+		this,
+		SLOT(MultiStateCountChanged(const NumberVariable<int> &)));
 
-	auto typesLayout = new QHBoxLayout;
+	auto typesLayout = new QHBoxLayout();
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{types}}", _types},
 	};
-	placeWidgets(obs_module_text(
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.condition.macro.type.selection"),
 		     typesLayout, widgetPlaceholders);
 
-	auto mainLayout = new QVBoxLayout;
+	auto mainLayout = new QVBoxLayout(this);
 	mainLayout->addLayout(typesLayout);
 	mainLayout->addLayout(_settingsLine1);
 	mainLayout->addLayout(_settingsLine2);
@@ -244,11 +264,13 @@ MacroConditionMacroEdit::MacroConditionMacroEdit(
 
 	_entryData = entryData;
 
-	connect(&_countTimer, SIGNAL(timeout()), this, SLOT(UpdateCount()));
+	QWidget::connect(&_countTimer, SIGNAL(timeout()), this,
+			 SLOT(UpdateCount()));
 	_countTimer.start(1000);
 
 	_pausedWarning->setVisible(false);
-	connect(&_pausedTimer, SIGNAL(timeout()), this, SLOT(UpdatePaused()));
+	QWidget::connect(&_pausedTimer, SIGNAL(timeout()), this,
+			 SLOT(UpdatePaused()));
 	_pausedTimer.start(1000);
 
 	UpdateEntryData();
@@ -264,42 +286,41 @@ void MacroConditionMacroEdit::ClearLayouts()
 	_settingsLine2->removeWidget(_resetCount);
 	_settingsLine1->removeWidget(_multiStateConditions);
 	_settingsLine1->removeWidget(_multiStateCount);
-	clearLayout(_settingsLine1);
-	clearLayout(_settingsLine2);
+	ClearLayout(_settingsLine1);
+	ClearLayout(_settingsLine2);
 }
 
 void MacroConditionMacroEdit::SetupStateWidgets()
 {
+	SetWidgetVisibility();
 	ClearLayouts();
 
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{macros}}", _macros},
 	};
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text("AdvSceneSwitcher.condition.macro.state.entry"),
 		_settingsLine1, widgetPlaceholders);
-	SetWidgetVisibility();
-	adjustSize();
 }
 
 void MacroConditionMacroEdit::SetupMultiStateWidgets()
 {
+	SetWidgetVisibility();
 	ClearLayouts();
 
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{multiStateConditions}}", _multiStateConditions},
 		{"{{multiStateCount}}", _multiStateCount},
 	};
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.condition.macro.multistate.entry"),
 		_settingsLine1, widgetPlaceholders);
-	SetWidgetVisibility();
-	adjustSize();
 }
 
 void MacroConditionMacroEdit::SetupCountWidgets()
 {
+	SetWidgetVisibility();
 	ClearLayouts();
 
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
@@ -309,16 +330,14 @@ void MacroConditionMacroEdit::SetupCountWidgets()
 		{"{{currentCount}}", _currentCount},
 		{"{{resetCount}}", _resetCount},
 	};
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.condition.macro.count.entry.line1"),
 		_settingsLine1, widgetPlaceholders);
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.condition.macro.count.entry.line2"),
 		_settingsLine2, widgetPlaceholders);
-	SetWidgetVisibility();
-	adjustSize();
 }
 
 void MacroConditionMacroEdit::SetWidgetVisibility()
@@ -329,7 +348,6 @@ void MacroConditionMacroEdit::SetWidgetVisibility()
 		_counterConditions->show();
 		_count->show();
 		_currentCount->show();
-		_pausedWarning->show();
 		_resetCount->show();
 		_macroList->hide();
 		_multiStateConditions->hide();
@@ -340,7 +358,6 @@ void MacroConditionMacroEdit::SetWidgetVisibility()
 		_counterConditions->hide();
 		_count->hide();
 		_currentCount->hide();
-		_pausedWarning->show();
 		_resetCount->hide();
 		_macroList->hide();
 		_multiStateConditions->hide();
@@ -360,6 +377,8 @@ void MacroConditionMacroEdit::SetWidgetVisibility()
 	default:
 		break;
 	}
+	adjustSize();
+	updateGeometry();
 }
 
 void MacroConditionMacroEdit::UpdateEntryData()
@@ -368,8 +387,7 @@ void MacroConditionMacroEdit::UpdateEntryData()
 		return;
 	}
 
-	auto test = _entryData->_type;
-	switch (test) {
+	switch (_entryData->_type) {
 	case MacroConditionMacro::Type::COUNT:
 		SetupCountWidgets();
 		break;
@@ -383,15 +401,15 @@ void MacroConditionMacroEdit::UpdateEntryData()
 		break;
 	}
 
-	_macros->SetCurrentMacro(_entryData->_macro.get());
+	_macros->SetCurrentMacro(_entryData->_macro);
 	_types->setCurrentIndex(static_cast<int>(_entryData->_type));
 	_counterConditions->setCurrentIndex(
 		static_cast<int>(_entryData->_counterCondition));
-	_count->setValue(_entryData->_count);
+	_count->SetValue(_entryData->_count);
 	_macroList->SetContent(_entryData->_macros);
 	_multiStateConditions->setCurrentIndex(
 		static_cast<int>(_entryData->_multiSateCondition));
-	_multiStateCount->setValue(_entryData->_multiSateCount);
+	_multiStateCount->SetValue(_entryData->_multiSateCount);
 }
 
 void MacroConditionMacroEdit::MacroChanged(const QString &text)
@@ -400,19 +418,19 @@ void MacroConditionMacroEdit::MacroChanged(const QString &text)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_macro.UpdateRef(text);
+	auto lock = LockContext();
+	_entryData->_macro = text;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
 }
 
-void MacroConditionMacroEdit::CountChanged(int value)
+void MacroConditionMacroEdit::CountChanged(const NumberVariable<int> &value)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_count = value;
 }
 
@@ -422,7 +440,7 @@ void MacroConditionMacroEdit::CountConditionChanged(int cond)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_counterCondition =
 		static_cast<MacroConditionMacro::CounterCondition>(cond);
 }
@@ -433,12 +451,9 @@ void MacroConditionMacroEdit::MacroRemove(const QString &)
 		return;
 	}
 
-	_entryData->_macro.UpdateRef();
-
 	auto it = _entryData->_macros.begin();
 	while (it != _entryData->_macros.end()) {
-		it->UpdateRef();
-		if (it->get() == nullptr) {
+		if (!it->GetMacro()) {
 			it = _entryData->_macros.erase(it);
 		} else {
 			++it;
@@ -453,7 +468,7 @@ void MacroConditionMacroEdit::TypeChanged(int type)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_type = static_cast<MacroConditionMacro::Type>(type);
 
 	switch (_entryData->_type) {
@@ -473,18 +488,26 @@ void MacroConditionMacroEdit::TypeChanged(int type)
 
 void MacroConditionMacroEdit::ResetClicked()
 {
-	if (_loading || !_entryData || !_entryData->_macro.get()) {
+	if (_loading || !_entryData) {
 		return;
 	}
 
-	_entryData->_macro->ResetRunCount();
+	auto macro = _entryData->_macro.GetMacro();
+	if (!macro) {
+		return;
+	}
+	macro->ResetRunCount();
 }
 
 void MacroConditionMacroEdit::UpdateCount()
 {
-	if (_entryData && _entryData->_macro.get()) {
-		_currentCount->setText(
-			QString::number(_entryData->_macro->RunCount()));
+	if (!_entryData) {
+		return;
+	}
+
+	auto macro = _entryData->_macro.GetMacro();
+	if (macro) {
+		_currentCount->setText(QString::number(macro->RunCount()));
 	} else {
 		_currentCount->setText("-");
 	}
@@ -492,10 +515,11 @@ void MacroConditionMacroEdit::UpdateCount()
 
 void MacroConditionMacroEdit::UpdatePaused()
 {
+	auto macro = _entryData->_macro.GetMacro();
 	_pausedWarning->setVisible(
 		_entryData &&
 		_entryData->_type != MacroConditionMacro::Type::MULTI_STATE &&
-		_entryData->_macro.get() && _entryData->_macro->Paused());
+		macro && macro->Paused());
 	adjustSize();
 }
 
@@ -505,18 +529,19 @@ void MacroConditionMacroEdit::MultiStateConditionChanged(int cond)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_multiSateCondition =
 		static_cast<MacroConditionMacro::MultiStateCondition>(cond);
 }
 
-void MacroConditionMacroEdit::MultiStateCountChanged(int value)
+void MacroConditionMacroEdit::MultiStateCountChanged(
+	const NumberVariable<int> &value)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_multiSateCount = value;
 }
 
@@ -526,7 +551,7 @@ void MacroConditionMacroEdit::Add(const std::string &name)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	MacroRef macro(name);
 	_entryData->_macros.push_back(macro);
 	adjustSize();
@@ -538,7 +563,7 @@ void MacroConditionMacroEdit::Remove(int idx)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_macros.erase(std::next(_entryData->_macros.begin(), idx));
 	adjustSize();
 }
@@ -550,7 +575,9 @@ void MacroConditionMacroEdit::Replace(int idx, const std::string &name)
 	}
 
 	MacroRef macro(name);
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_macros[idx] = macro;
 	adjustSize();
 }
+
+} // namespace advss
