@@ -1,7 +1,8 @@
-#include "macro-condition-edit.hpp"
 #include "macro-condition-plugin-state.hpp"
+#include "switcher-data.hpp"
 #include "utility.hpp"
-#include "advanced-scene-switcher.hpp"
+
+namespace advss {
 
 const uint32_t MacroConditionPluginState::_version = 1;
 
@@ -123,7 +124,7 @@ MacroConditionPluginStateEdit::MacroConditionPluginStateEdit(
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{condition}}", _condition},
 	};
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text("AdvSceneSwitcher.condition.pluginState.entry"),
 		switchLayout, widgetPlaceholders);
 
@@ -142,7 +143,7 @@ void MacroConditionPluginStateEdit::ConditionChanged(int idx)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	if (_entryData->_condition ==
 	    MacroConditionPluginState::Condition::OBS_SHUTDOWN) {
 		switcher->shutdownConditionCount--;
@@ -165,3 +166,5 @@ void MacroConditionPluginStateEdit::UpdateEntryData()
 	_condition->setCurrentIndex(
 		_condition->findData(static_cast<int>(_entryData->_condition)));
 }
+
+} // namespace advss
