@@ -1,7 +1,7 @@
-#include "macro-condition-edit.hpp"
 #include "macro-condition-scene-transform.hpp"
 #include "utility.hpp"
-#include "advanced-scene-switcher.hpp"
+
+namespace advss {
 
 const std::string MacroConditionSceneTransform::id = "scene_transform";
 
@@ -17,13 +17,15 @@ bool MacroConditionSceneTransform::CheckCondition()
 	bool ret = false;
 	auto items = _source.GetSceneItems(_scene);
 
+	std::string json;
 	for (auto &item : items) {
-		auto json = getSceneItemTransform(item);
-		if (matchJson(json, _settings, _regex)) {
+		json = GetSceneItemTransform(item);
+		if (MatchJson(json, _settings, _regex)) {
 			ret = true;
 		}
 		obs_sceneitem_release(item);
 	}
+	SetVariableValue(json);
 	return ret;
 }
 
@@ -101,17 +103,17 @@ MacroConditionSceneTransformEdit::MacroConditionSceneTransformEdit(
 	};
 
 	QHBoxLayout *line1Layout = new QHBoxLayout;
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.condition.sceneTransform.entry.line1"),
 		line1Layout, widgetPlaceholders);
 	QHBoxLayout *line2Layout = new QHBoxLayout;
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.condition.sceneTransform.entry.line2"),
 		line2Layout, widgetPlaceholders, false);
 	QHBoxLayout *line3Layout = new QHBoxLayout;
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.condition.sceneTransform.entry.line3"),
 		line3Layout, widgetPlaceholders);
@@ -148,7 +150,7 @@ void MacroConditionSceneTransformEdit::SceneChanged(const SceneSelection &s)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_scene = s;
 }
 
@@ -159,7 +161,7 @@ void MacroConditionSceneTransformEdit::SourceChanged(
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_source = item;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -177,9 +179,9 @@ void MacroConditionSceneTransformEdit::GetSettingsClicked()
 		return;
 	}
 
-	auto settings = formatJsonString(getSceneItemTransform(items[0]));
+	auto settings = FormatJsonString(GetSceneItemTransform(items[0]));
 	if (_entryData->_regex.Enabled()) {
-		settings = escapeForRegex(settings);
+		settings = EscapeForRegex(settings);
 	}
 	_settings->setPlainText(settings);
 
@@ -194,7 +196,7 @@ void MacroConditionSceneTransformEdit::SettingsChanged()
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_settings = _settings->toPlainText().toStdString();
 
 	adjustSize();
@@ -207,9 +209,11 @@ void MacroConditionSceneTransformEdit::RegexChanged(RegexConfig conf)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_regex = conf;
 
 	adjustSize();
 	updateGeometry();
 }
+
+} // namespace advss
