@@ -1,7 +1,7 @@
-#include "macro-condition-edit.hpp"
 #include "macro-condition-studio-mode.hpp"
 #include "utility.hpp"
-#include "advanced-scene-switcher.hpp"
+
+namespace advss {
 
 const std::string MacroConditionStudioMode::id = "studio_mode";
 
@@ -100,7 +100,7 @@ MacroConditionStudioModeEdit::MacroConditionStudioModeEdit(
 		{"{{conditions}}", _condition},
 		{"{{scenes}}", _scenes},
 	};
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text("AdvSceneSwitcher.condition.studioMode.entry"),
 		layout, widgetPlaceholders);
 	setLayout(layout);
@@ -116,7 +116,7 @@ void MacroConditionStudioModeEdit::ConditionChanged(int cond)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_condition = static_cast<StudioModeCondition>(cond);
 	SetWidgetVisibility();
 	emit HeaderInfoChanged(
@@ -129,7 +129,7 @@ void MacroConditionStudioModeEdit::SceneChanged(const SceneSelection &s)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_scene = s;
 	emit HeaderInfoChanged(
 		QString::fromStdString(_entryData->GetShortDesc()));
@@ -155,3 +155,5 @@ void MacroConditionStudioModeEdit::SetWidgetVisibility()
 	_scenes->setVisible(_entryData->_condition ==
 			    StudioModeCondition::PREVIEW_SCENE);
 }
+
+} // namespace advss
