@@ -1,7 +1,8 @@
-#include "macro-condition-edit.hpp"
 #include "macro-condition-transition.hpp"
+#include "switcher-data.hpp"
 #include "utility.hpp"
-#include "advanced-scene-switcher.hpp"
+
+namespace advss {
 
 const std::string MacroConditionTransition::id = "transition";
 
@@ -25,7 +26,7 @@ static std::map<TransitionCondition, std::string> filterConditionTypes = {
 	 "AdvSceneSwitcher.condition.transition.type.transitionTarget"},
 };
 
-bool isCurrentTransition(OBSWeakSource &t)
+static bool isCurrentTransition(OBSWeakSource &t)
 {
 	bool match;
 	auto tSource = obs_frontend_get_current_transition();
@@ -36,7 +37,7 @@ bool isCurrentTransition(OBSWeakSource &t)
 	return match;
 }
 
-bool isTargetScene(OBSWeakSource &target)
+static bool isTargetScene(OBSWeakSource &target)
 {
 	auto source = obs_frontend_get_current_scene();
 	auto targetScene = obs_source_get_weak_source(source);
@@ -50,7 +51,7 @@ bool MacroConditionTransition::CheckCondition()
 {
 	bool anyTransitionEnded = _lastTransitionEndTime !=
 				  switcher->lastTransitionEndTime;
-	bool anyTransitionStarted = switcher->anySceneTransitionStarted();
+	bool anyTransitionStarted = switcher->AnySceneTransitionStarted();
 	bool transitionStarted = false;
 	bool transitionEnded = false;
 
@@ -70,7 +71,7 @@ bool MacroConditionTransition::CheckCondition()
 		break;
 	}
 	case TransitionCondition::DURATION:
-		ret = _duration.seconds * 1000 ==
+		ret = _duration.Milliseconds() ==
 		      obs_frontend_get_transition_duration();
 		break;
 	case TransitionCondition::STARTED:
@@ -198,8 +199,8 @@ MacroConditionTransitionEdit::MacroConditionTransitionEdit(
 			 SLOT(TransitionChanged(const TransitionSelection &)));
 	QWidget::connect(_scenes, SIGNAL(SceneChanged(const SceneSelection &)),
 			 this, SLOT(SceneChanged(const SceneSelection &)));
-	QWidget::connect(_duration, SIGNAL(DurationChanged(double)), this,
-			 SLOT(DurationChanged(double)));
+	QWidget::connect(_duration, SIGNAL(DurationChanged(const Duration &)),
+			 this, SLOT(DurationChanged(const Duration &)));
 
 	QHBoxLayout *mainLayout = new QHBoxLayout;
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
@@ -209,7 +210,7 @@ MacroConditionTransitionEdit::MacroConditionTransitionEdit(
 		{"{{duration}}", _duration},
 		{"{{durationSuffix}}", _durationSuffix},
 	};
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text("AdvSceneSwitcher.condition.transition.entry"),
 		mainLayout, widgetPlaceholders);
 	setLayout(mainLayout);
@@ -226,7 +227,7 @@ void MacroConditionTransitionEdit::ConditionChanged(int index)
 	}
 
 	{
-		std::lock_guard<std::mutex> lock(switcher->m);
+		auto lock = LockContext();
 		_entryData->_condition =
 			static_cast<TransitionCondition>(index);
 	}
@@ -242,7 +243,7 @@ void MacroConditionTransitionEdit::TransitionChanged(
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->DisconnectTransitionSignals();
 	_entryData->_transition = t;
 	_entryData->ConnectToTransitionSignals();
@@ -256,18 +257,18 @@ void MacroConditionTransitionEdit::SceneChanged(const SceneSelection &s)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_scene = s;
 }
 
-void MacroConditionTransitionEdit::DurationChanged(double seconds)
+void MacroConditionTransitionEdit::DurationChanged(const Duration &dur)
 {
 	if (_loading || !_entryData) {
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_duration.seconds = seconds;
+	auto lock = LockContext();
+	_entryData->_duration = dur;
 }
 
 void MacroConditionTransitionEdit::SetWidgetVisibility()
@@ -307,3 +308,5 @@ void MacroConditionTransitionEdit::UpdateEntryData()
 	_scenes->SetScene(_entryData->_scene);
 	_duration->SetDuration(_entryData->_duration);
 }
+
+} // namespace advss
