@@ -1,7 +1,7 @@
-#include "macro-condition-edit.hpp"
 #include "macro-condition-virtual-cam.hpp"
 #include "utility.hpp"
-#include "advanced-scene-switcher.hpp"
+
+namespace advss {
 
 const std::string MacroConditionVCam::id = "virtual_cam";
 
@@ -73,7 +73,7 @@ MacroConditionVCamEdit::MacroConditionVCamEdit(
 		{"{{states}}", _states},
 	};
 
-	placeWidgets(obs_module_text(
+	PlaceWidgets(obs_module_text(
 			     "AdvSceneSwitcher.condition.virtualCamera.entry"),
 		     mainLayout, widgetPlaceholders);
 	setLayout(mainLayout);
@@ -89,7 +89,7 @@ void MacroConditionVCamEdit::StateChanged(int value)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_state = static_cast<VCamState>(value);
 }
 
@@ -101,3 +101,5 @@ void MacroConditionVCamEdit::UpdateEntryData()
 
 	_states->setCurrentIndex(static_cast<int>(_entryData->_state));
 }
+
+} // namespace advss
