@@ -1,10 +1,11 @@
-#include "macro-condition-edit.hpp"
 #include "macro-condition-websocket.hpp"
+#include "switcher-data.hpp"
 #include "utility.hpp"
-#include "advanced-scene-switcher.hpp"
 
 #include <regex>
 
+namespace advss {
+
 const std::string MacroConditionWebsocket::id = "websocket";
 
 bool MacroConditionWebsocket::_registered = MacroConditionFactory::Register(
@@ -38,7 +39,7 @@ bool MacroConditionWebsocket::CheckCondition()
 		messages = &switcher->websocketMessages;
 		break;
 	case MacroConditionWebsocket::Type::EVENT: {
-		auto connection = GetConnectionByName(_connection);
+		auto connection = _connection.lock();
 		if (!connection) {
 			return false;
 		}
@@ -76,7 +77,8 @@ bool MacroConditionWebsocket::Save(obs_data_t *obj) const
 	obs_data_set_int(obj, "type", static_cast<int>(_type));
 	_message.Save(obj, "message");
 	_regex.Save(obj);
-	obs_data_set_string(obj, "connection", _connection.c_str());
+	obs_data_set_string(obj, "connection",
+			    GetWeakConnectionName(_connection).c_str());
 	return true;
 }
 
@@ -91,7 +93,8 @@ bool MacroConditionWebsocket::Load(obs_data_t *obj)
 		_regex.CreateBackwardsCompatibleRegex(
 			obs_data_get_bool(obj, "useRegex"), false);
 	}
-	_connection = obs_data_get_string(obj, "connection");
+	_connection =
+		GetWeakConnectionByName(obs_data_get_string(obj, "connection"));
 	return true;
 }
 
@@ -100,7 +103,7 @@ std::string MacroConditionWebsocket::GetShortDesc() const
 	if (_type == Type::REQUEST) {
 		return "";
 	}
-	return _connection;
+	return GetWeakConnectionName(_connection);
 }
 
 static inline void populateConditionSelection(QComboBox *list)
@@ -150,12 +153,12 @@ void MacroConditionWebsocketEdit::SetupRequestEdit()
 {
 	_editLayout->removeWidget(_conditions);
 	_editLayout->removeWidget(_connection);
-	clearLayout(_editLayout);
+	ClearLayout(_editLayout);
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{type}}", _conditions},
 		{"{{connection}}", _connection},
 	};
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.condition.websocket.entry.request"),
 		_editLayout, widgetPlaceholders);
@@ -166,12 +169,12 @@ void MacroConditionWebsocketEdit::SetupEventEdit()
 {
 	_editLayout->removeWidget(_conditions);
 	_editLayout->removeWidget(_connection);
-	clearLayout(_editLayout);
+	ClearLayout(_editLayout);
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{type}}", _conditions},
 		{"{{connection}}", _connection},
 	};
-	placeWidgets(
+	PlaceWidgets(
 		obs_module_text(
 			"AdvSceneSwitcher.condition.websocket.entry.event"),
 		_editLayout, widgetPlaceholders);
@@ -205,7 +208,7 @@ void MacroConditionWebsocketEdit::ConditionChanged(int index)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_type = static_cast<MacroConditionWebsocket::Type>(index);
 	if (_entryData->_type == MacroConditionWebsocket::Type::REQUEST) {
 		SetupRequestEdit();
@@ -222,7 +225,7 @@ void MacroConditionWebsocketEdit::MessageChanged()
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_message = _message->toPlainText().toUtf8().constData();
 
 	adjustSize();
@@ -236,8 +239,8 @@ void MacroConditionWebsocketEdit::ConnectionSelectionChanged(
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
-	_entryData->_connection = connection.toStdString();
+	auto lock = LockContext();
+	_entryData->_connection = GetWeakConnectionByQString(connection);
 	emit(HeaderInfoChanged(connection));
 }
 
@@ -247,9 +250,11 @@ void MacroConditionWebsocketEdit::RegexChanged(RegexConfig conf)
 		return;
 	}
 
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	_entryData->_regex = conf;
 
 	adjustSize();
 	updateGeometry();
 }
+
+} // namespace advss
