@@ -1,10 +1,13 @@
 #include "macro-selection.hpp"
 #include "advanced-scene-switcher.hpp"
+#include "switcher-data.hpp"
 #include "utility.hpp"
 
 #include <QStandardItemModel>
 #include <QDialogButtonBox>
 
+namespace advss {
+
 MacroSelection::MacroSelection(QWidget *parent) : QComboBox(parent)
 {
 	addItem(obs_module_text("AdvSceneSwitcher.selectMacro"));
@@ -39,8 +42,9 @@ void MacroSelection::MacroAdd(const QString &name)
 	addItem(name);
 }
 
-void MacroSelection::SetCurrentMacro(Macro *m)
+void MacroSelection::SetCurrentMacro(const MacroRef &macro)
 {
+	auto m = macro.GetMacro();
 	if (!m) {
 		this->setCurrentIndex(0);
 	} else {
@@ -63,8 +67,8 @@ void MacroSelection::HideSelectedMacro()
 	if (idx == -1) {
 		return;
 	}
-	// +1 for "select macro" entry
-	qobject_cast<QListView *>(view())->setRowHidden(idx + 1, true);
+
+	qobject_cast<QListView *>(view())->setRowHidden(idx, true);
 }
 
 void MacroSelection::ShowAllMacros()
@@ -119,7 +123,7 @@ MacroSelectionDialog::MacroSelectionDialog(QWidget *)
 	std::unordered_map<std::string, QWidget *> widgetPlaceholders = {
 		{"{{macroSelection}}", _macroSelection},
 	};
-	placeWidgets(obs_module_text("AdvSceneSwitcher.askForMacro"),
+	PlaceWidgets(obs_module_text("AdvSceneSwitcher.askForMacro"),
 		     selectionLayout, widgetPlaceholders);
 	auto *layout = new QVBoxLayout();
 	layout->addLayout(selectionLayout);
@@ -142,3 +146,5 @@ bool MacroSelectionDialog::AskForMacro(QWidget *parent, std::string &macroName)
 
 	return true;
 }
+
+} // namespace advss
