@@ -2,14 +2,17 @@
 #include <QComboBox>
 #include <QDialog>
 
+namespace advss {
+
 class Macro;
+class MacroRef;
 
 class MacroSelection : public QComboBox {
 	Q_OBJECT
 
 public:
 	MacroSelection(QWidget *parent);
-	void SetCurrentMacro(Macro *);
+	void SetCurrentMacro(const MacroRef &);
 	void HideSelectedMacro(); // Macro currently being edited
 	void ShowAllMacros();
 
@@ -29,3 +32,5 @@ public:
 private:
 	MacroSelection *_macroSelection;
 };
+
+} // namespace advss
