@@ -3,6 +3,7 @@
 #include "macro-action-edit.hpp"
 #include "macro-condition-edit.hpp"
 #include "advanced-scene-switcher.hpp"
+#include "switcher-data.hpp"
 #include "name-dialog.hpp"
 #include "macro-properties.hpp"
 #include "utility.hpp"
@@ -12,15 +13,17 @@
 #include <QGraphicsOpacityEffect>
 #include <QPropertyAnimation>
 
+namespace advss {
+
 static QMetaObject::Connection addPulse;
 static QTimer onChangeHighlightTimer;
 
-bool macroNameExists(std::string name)
+static bool macroNameExists(std::string name)
 {
 	return !!GetMacroByName(name.c_str());
 }
 
-bool AdvSceneSwitcher::addNewMacro(std::shared_ptr<Macro> &res,
+bool AdvSceneSwitcher::AddNewMacro(std::shared_ptr<Macro> &res,
 				   std::string &name, std::string format)
 {
 	QString fmt;
@@ -58,7 +61,7 @@ bool AdvSceneSwitcher::addNewMacro(std::shared_ptr<Macro> &res,
 	}
 
 	{
-		std::lock_guard<std::mutex> lock(switcher->m);
+		auto lock = LockContext();
 		res = std::make_shared<Macro>(
 			name,
 			switcher->macroProperties._newMacroRegisterHotkeys);
@@ -70,12 +73,12 @@ void AdvSceneSwitcher::on_macroAdd_clicked()
 {
 	std::string name;
 	std::shared_ptr<Macro> newMacro;
-	if (!addNewMacro(newMacro, name)) {
+	if (!AddNewMacro(newMacro, name)) {
 		return;
 	}
 
 	{
-		std::lock_guard<std::mutex> lock(switcher->m);
+		auto lock = LockContext();
 		ui->macros->Add(newMacro);
 	}
 
@@ -83,14 +86,14 @@ void AdvSceneSwitcher::on_macroAdd_clicked()
 	emit MacroAdded(QString::fromStdString(name));
 }
 
-void AdvSceneSwitcher::on_macroRemove_clicked()
+void AdvSceneSwitcher::RemoveMacro(std::shared_ptr<Macro> &macro)
 {
-	auto macro = getSelectedMacro();
 	if (!macro) {
 		return;
 	}
+
 	auto name = QString::fromStdString(macro->Name());
-	if (macro->IsGroup()) {
+	if (macro->IsGroup() && macro->GroupSize() > 0) {
 		QString deleteWarning = obs_module_text(
 			"AdvSceneSwitcher.macroTab.groupDeleteConfirm");
 		if (!DisplayMessage(deleteWarning.arg(name), true)) {
@@ -99,17 +102,58 @@ void AdvSceneSwitcher::on_macroRemove_clicked()
 	}
 
 	{
-		std::lock_guard<std::mutex> lock(switcher->m);
+		auto lock = LockContext();
 		ui->macros->Remove(macro);
 	}
 
 	emit MacroRemoved(name);
 }
 
+void AdvSceneSwitcher::RenameMacro(std::shared_ptr<Macro> &macro,
+				   const QString &name)
+{
+	auto oldName = QString::fromStdString(macro->Name());
+	{
+		auto lock = LockContext();
+		macro->SetName(name.toStdString());
+	}
+	emit MacroRenamed(oldName, name);
+}
+
+void AdvSceneSwitcher::on_macroRemove_clicked()
+{
+	auto macros = GetSelectedMacros();
+	if (macros.empty()) {
+		return;
+	}
+
+	if (macros.size() == 1) {
+		QString deleteWarning = obs_module_text(
+			"AdvSceneSwitcher.macroTab.deleteSingleMacroConfirmation");
+		if (!DisplayMessage(deleteWarning.arg(QString::fromStdString(
+					    macros.at(0)->Name())),
+				    true)) {
+			return;
+		}
+		RemoveMacro(macros.at(0));
+		return;
+	}
+
+	QString deleteWarning = obs_module_text(
+		"AdvSceneSwitcher.macroTab.deleteMultipleMacrosConfirmation");
+	if (!DisplayMessage(deleteWarning.arg(macros.size()), true)) {
+		return;
+	}
+
+	for (auto &macro : macros) {
+		RemoveMacro(macro);
+	}
+}
+
 void AdvSceneSwitcher::on_macroUp_clicked()
 {
-	std::lock_guard<std::mutex> lock(switcher->m);
-	auto macro = getSelectedMacro();
+	auto lock = LockContext();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -118,19 +162,49 @@ void AdvSceneSwitcher::on_macroUp_clicked()
 
 void AdvSceneSwitcher::on_macroDown_clicked()
 {
-	std::lock_guard<std::mutex> lock(switcher->m);
-	auto macro = getSelectedMacro();
+	auto lock = LockContext();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
 	ui->macros->Down(macro);
 }
 
+static bool newMacroNameValid(const std::string &name)
+{
+	if (!macroNameExists(name)) {
+		return true;
+	}
+	DisplayMessage(obs_module_text("AdvSceneSwitcher.macroTab.exists"));
+	return false;
+}
+
+void AdvSceneSwitcher::RenameCurrentMacro()
+{
+	auto macro = GetSelectedMacro();
+	if (!macro) {
+		return;
+	}
+	std::string oldName = macro->Name();
+	std::string name;
+	if (!AdvSSNameDialog::AskForName(
+		    this, obs_module_text("AdvSceneSwitcher.windowTitle"),
+		    obs_module_text("AdvSceneSwitcher.item.newName"), name,
+		    QString::fromStdString(oldName))) {
+		return;
+	}
+	if (name.empty() || name == oldName || !newMacroNameValid(name)) {
+		return;
+	}
+	RenameMacro(macro, QString::fromStdString(name));
+
+	const QSignalBlocker b(ui->macroName);
+	ui->macroName->setText(QString::fromStdString(name));
+}
+
 void AdvSceneSwitcher::on_macroName_editingFinished()
 {
-	bool nameValid = true;
-
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -138,38 +212,17 @@ void AdvSceneSwitcher::on_macroName_editingFinished()
 	QString newName = ui->macroName->text();
 	QString oldName = QString::fromStdString(macro->Name());
 
-	if (newName.isEmpty() || newName == oldName) {
-		nameValid = false;
-	}
-
-	if (nameValid && macroNameExists(newName.toUtf8().constData())) {
-		DisplayMessage(
-			obs_module_text("AdvSceneSwitcher.macroTab.exists"));
-		nameValid = false;
-	}
-
-	{
-		std::lock_guard<std::mutex> lock(switcher->m);
-		if (nameValid) {
-			macro->SetName(newName.toUtf8().constData());
-			auto macro = getSelectedMacro();
-			if (!macro) {
-				return;
-			}
-			macro->SetName(newName.toStdString());
-		} else {
-			ui->macroName->setText(oldName);
-		}
-	}
-
-	if (nameValid) {
-		emit MacroRenamed(oldName, newName);
+	if (newName.isEmpty() || newName == oldName ||
+	    !newMacroNameValid(newName.toStdString())) {
+		ui->macroName->setText(oldName);
+		return;
 	}
+	RenameMacro(macro, newName);
 }
 
 void AdvSceneSwitcher::on_runMacro_clicked()
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -184,21 +237,21 @@ void AdvSceneSwitcher::on_runMacro_clicked()
 
 void AdvSceneSwitcher::on_runMacroInParallel_stateChanged(int value)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	macro->SetRunInParallel(value);
 }
 
 void AdvSceneSwitcher::on_runMacroOnChange_stateChanged(int value)
 {
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
-	std::lock_guard<std::mutex> lock(switcher->m);
+	auto lock = LockContext();
 	macro->SetMatchOnChange(value);
 }
 
@@ -309,11 +362,16 @@ void AdvSceneSwitcher::HighlightCondition(int idx, QColor color)
 	conditionsList->Highlight(idx, color);
 }
 
-std::shared_ptr<Macro> AdvSceneSwitcher::getSelectedMacro()
+std::shared_ptr<Macro> AdvSceneSwitcher::GetSelectedMacro()
 {
 	return ui->macros->GetCurrentMacro();
 }
 
+std::vector<std::shared_ptr<Macro>> AdvSceneSwitcher::GetSelectedMacros()
+{
+	return ui->macros->GetCurrentMacros();
+}
+
 void AdvSceneSwitcher::MacroSelectionChanged(const QItemSelection &,
 					     const QItemSelection &)
 {
@@ -321,7 +379,7 @@ void AdvSceneSwitcher::MacroSelectionChanged(const QItemSelection &,
 		return;
 	}
 
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		SetMacroEditAreaDisabled(true);
 		conditionsList->Clear();
@@ -340,7 +398,7 @@ void AdvSceneSwitcher::HighlightOnChange()
 		return;
 	}
 
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -355,7 +413,7 @@ void AdvSceneSwitcher::on_macroProperties_clicked()
 {
 	MacroProperties prop = switcher->macroProperties;
 	bool accepted = MacroPropertiesDialog::AskForSettings(
-		this, prop, getSelectedMacro().get());
+		this, prop, GetSelectedMacro().get());
 	if (!accepted) {
 		return;
 	}
@@ -380,7 +438,7 @@ bool shouldResotreSplitterPos(const QList<int> &pos)
 	return true;
 }
 
-void AdvSceneSwitcher::setupMacroTab()
+void AdvSceneSwitcher::SetupMacroTab()
 {
 	if (switcher->macros.size() == 0 && !switcher->disableHints) {
 		addPulse = PulseWidget(ui->macroAdd, QColor(Qt::green));
@@ -467,15 +525,21 @@ void AdvSceneSwitcher::ShowMacroContextMenu(const QPoint &pos)
 {
 	QPoint globalPos = ui->macros->mapToGlobal(pos);
 	QMenu menu;
+
+	menu.addAction(
+		obs_module_text("AdvSceneSwitcher.macroTab.contextMenuAdd"),
+		this, &AdvSceneSwitcher::on_macroAdd_clicked);
+
 	auto copy = menu.addAction(
 		obs_module_text("AdvSceneSwitcher.macroTab.copy"), this,
 		&AdvSceneSwitcher::CopyMacro);
-	copy->setDisabled(ui->macros->GroupsSelected());
+	copy->setEnabled(ui->macros->SingleItemSelected() &&
+			 !ui->macros->GroupsSelected());
+	menu.addSeparator();
 
 	auto group = menu.addAction(
 		obs_module_text("AdvSceneSwitcher.macroTab.group"), ui->macros,
 		&MacroTree::GroupSelectedItems);
-	// Nested groups are not supported
 	group->setDisabled(ui->macros->GroupedItemsSelected() ||
 			   ui->macros->GroupsSelected() ||
 			   ui->macros->SelectionEmpty());
@@ -484,6 +548,17 @@ void AdvSceneSwitcher::ShowMacroContextMenu(const QPoint &pos)
 		obs_module_text("AdvSceneSwitcher.macroTab.ungroup"),
 		ui->macros, &MacroTree::UngroupSelectedGroups);
 	ungroup->setEnabled(ui->macros->GroupsSelected());
+	menu.addSeparator();
+
+	auto rename = menu.addAction(
+		obs_module_text("AdvSceneSwitcher.macroTab.rename"), this,
+		&AdvSceneSwitcher::RenameCurrentMacro);
+	rename->setEnabled(ui->macros->SingleItemSelected());
+
+	auto remove = menu.addAction(
+		obs_module_text("AdvSceneSwitcher.macroTab.remove"), this,
+		&AdvSceneSwitcher::on_macroRemove_clicked);
+	remove->setDisabled(ui->macros->SelectionEmpty());
 
 	menu.exec(globalPos);
 }
@@ -520,7 +595,7 @@ void AdvSceneSwitcher::ShowMacroConditionsContextMenu(const QPoint &pos)
 
 void AdvSceneSwitcher::CopyMacro()
 {
-	const auto macro = getSelectedMacro();
+	const auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -528,24 +603,26 @@ void AdvSceneSwitcher::CopyMacro()
 	std::string format = macro->Name() + " %1";
 	std::string name;
 	std::shared_ptr<Macro> newMacro;
-	if (!addNewMacro(newMacro, name, format)) {
+	if (!AddNewMacro(newMacro, name, format)) {
 		return;
 	}
 
 	obs_data_t *data = obs_data_create();
 	macro->Save(data);
 	newMacro->Load(data);
+	newMacro->PostLoad();
 	newMacro->SetName(name);
+	Macro::PrepareMoveToGroup(macro->Parent(), newMacro);
 	obs_data_release(data);
 
-	ui->macros->Add(newMacro);
+	ui->macros->Add(newMacro, macro);
 	ui->macroAdd->disconnect(addPulse);
 	emit MacroAdded(QString::fromStdString(name));
 }
 
 void AdvSceneSwitcher::ExpandAllActions()
 {
-	auto m = getSelectedMacro();
+	auto m = GetSelectedMacro();
 	if (!m) {
 		return;
 	}
@@ -554,7 +631,7 @@ void AdvSceneSwitcher::ExpandAllActions()
 
 void AdvSceneSwitcher::ExpandAllConditions()
 {
-	auto m = getSelectedMacro();
+	auto m = GetSelectedMacro();
 	if (!m) {
 		return;
 	}
@@ -563,7 +640,7 @@ void AdvSceneSwitcher::ExpandAllConditions()
 
 void AdvSceneSwitcher::CollapseAllActions()
 {
-	auto m = getSelectedMacro();
+	auto m = GetSelectedMacro();
 	if (!m) {
 		return;
 	}
@@ -572,7 +649,7 @@ void AdvSceneSwitcher::CollapseAllActions()
 
 void AdvSceneSwitcher::CollapseAllConditions()
 {
-	auto m = getSelectedMacro();
+	auto m = GetSelectedMacro();
 	if (!m) {
 		return;
 	}
@@ -611,7 +688,7 @@ void AdvSceneSwitcher::UpMacroSegementHotkey()
 		return;
 	}
 
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -667,7 +744,7 @@ void AdvSceneSwitcher::DownMacroSegementHotkey()
 		return;
 	}
 
-	auto macro = getSelectedMacro();
+	auto macro = GetSelectedMacro();
 	if (!macro) {
 		return;
 	}
@@ -730,7 +807,7 @@ void AdvSceneSwitcher::DeleteMacroSegementHotkey()
 	}
 }
 
-void fade(QWidget *widget, bool fadeOut)
+static void fade(QWidget *widget, bool fadeOut)
 {
 	const double fadeOutOpacity = 0.3;
 	// Don't use exactly 1.0 as for some reason this causes buttons in
@@ -742,9 +819,9 @@ void fade(QWidget *widget, bool fadeOut)
 		auto curOpacity =
 			dynamic_cast<QGraphicsOpacityEffect *>(curEffect);
 		if (curOpacity &&
-		    ((fadeOut && doubleEquals(curOpacity->opacity(),
+		    ((fadeOut && DoubleEquals(curOpacity->opacity(),
 					      fadeOutOpacity, 0.0001)) ||
-		     (!fadeOut && doubleEquals(curOpacity->opacity(),
+		     (!fadeOut && DoubleEquals(curOpacity->opacity(),
 					       fadeInOpacity, 0.0001)))) {
 			return;
 		}
@@ -809,3 +886,5 @@ void AdvSceneSwitcher::HighlightControls()
 		ResetOpacityConditionControls();
 	}
 }
+
+} // namespace advss
