@@ -1,7 +1,7 @@
 #include "macro-tree.hpp"
 #include "macro.hpp"
 #include "utility.hpp"
-#include "switcher-data-structs.hpp"
+#include "switcher-data.hpp"
 
 #include <obs.h>
 #include <string>
@@ -14,6 +14,10 @@
 #include <QMouseEvent>
 #include <QStylePainter>
 
+Q_DECLARE_METATYPE(std::shared_ptr<advss::Macro>);
+
+namespace advss {
+
 MacroTreeItem::MacroTreeItem(MacroTree *tree, std::shared_ptr<Macro> macroItem,
 			     bool highlight)
 	: _tree(tree), _highlight(highlight), _macro(macroItem)
@@ -27,10 +31,9 @@ MacroTreeItem::MacroTreeItem(MacroTree *tree, std::shared_ptr<Macro> macroItem,
 	bool isGroup = _macro->IsGroup();
 
 	if (isGroup) {
-		QIcon icon;
-		icon.addFile(
-			QString::fromUtf8(":/res/images/sources/group.svg"),
-			QSize(), QIcon::Normal, QIcon::Off);
+		const auto path = QString::fromStdString(GetDataFilePath(
+			"res/images/" + GetThemeTypeName() + "Group.svg"));
+		QIcon icon(path);
 		QPixmap pixmap = icon.pixmap(QSize(16, 16));
 		_iconLabel = new QLabel();
 		_iconLabel->setPixmap(pixmap);
@@ -232,6 +235,10 @@ static inline int
 MacroIndexToModelIndex(int realIdx,
 		       const std::deque<std::shared_ptr<Macro>> &macros)
 {
+	if (realIdx == -1) {
+		return -1;
+	}
+
 	int modelIdx = 0;
 	bool inCollapsedGroup = false;
 	uint32_t groupSize = 0;
@@ -242,7 +249,7 @@ MacroIndexToModelIndex(int realIdx,
 			inCollapsedGroup = false;
 			continue;
 		}
-		const auto &m = macros[i];
+		const auto &m = macros.at(i);
 		inCollapsedGroup = m->IsGroup() && m->IsCollapsed();
 		groupSize = m->GroupSize();
 		modelIdx++;
@@ -253,15 +260,25 @@ MacroIndexToModelIndex(int realIdx,
 void MacroTreeModel::MoveItemBefore(const std::shared_ptr<Macro> &item,
 				    const std::shared_ptr<Macro> &before)
 {
-	if (item == before) {
+	if (!item || !before || item == before ||
+	    Neighbor(item, false) == before) {
 		return;
 	}
 
-	auto modelFromEndIdx = GetItemModelIndex(item);
-	auto modelFromIdx = modelFromEndIdx;
-	auto modelTo = GetItemModelIndex(before);
-	auto macroFrom = GetItemMacroIndex(item);
-	auto macroTo = GetItemMacroIndex(before);
+	int modelFromIdx = 0, modelFromEndIdx = 0, modelTo = 0, macroFrom = 0,
+	    macroTo = 0;
+	try {
+		modelFromEndIdx = GetItemModelIndex(item);
+		modelFromIdx = modelFromEndIdx;
+		modelTo = GetItemModelIndex(before);
+		macroFrom = GetItemMacroIndex(item);
+		macroTo = GetItemMacroIndex(before);
+	} catch (const std::out_of_range &) {
+		blog(LOG_WARNING,
+		     "error while trying to move item \"%s\" before \"%s\"",
+		     item->Name().c_str(), before->Name().c_str());
+		return;
+	}
 
 	if (before->IsSubitem()) {
 		modelTo -= before->IsCollapsed() ? 0 : before->GroupSize();
@@ -292,24 +309,29 @@ void MacroTreeModel::MoveItemBefore(const std::shared_ptr<Macro> &item,
 		_macros.insert(std::next(_macros.begin(), macroTo + i), tmp);
 	}
 	endMoveRows();
-
-	for (auto &m : _macros) {
-		m->ResolveMacroRef();
-	}
 }
 
 void MacroTreeModel::MoveItemAfter(const std::shared_ptr<Macro> &item,
 				   const std::shared_ptr<Macro> &after)
 {
-	if (item == after) {
+	if (!item || !after || item == after || Neighbor(item, true) == after) {
 		return;
 	}
 
-	auto modelFromIdx = GetItemModelIndex(item);
-	auto modelFromEndIdx = modelFromIdx;
-	auto modelTo = GetItemModelIndex(after);
-	auto macroFrom = GetItemMacroIndex(item);
-	auto macroTo = GetItemMacroIndex(after);
+	int modelFromIdx = 0, modelFromEndIdx = 0, modelTo = 0, macroFrom = 0,
+	    macroTo = 0;
+	try {
+		modelFromIdx = GetItemModelIndex(item);
+		modelFromEndIdx = modelFromIdx;
+		modelTo = GetItemModelIndex(after);
+		macroFrom = GetItemMacroIndex(item);
+		macroTo = GetItemMacroIndex(after);
+	} catch (const std::out_of_range &) {
+		blog(LOG_WARNING,
+		     "error while trying to move item \"%s\" after \"%s\"",
+		     item->Name().c_str(), after->Name().c_str());
+		return;
+	}
 
 	if (after->IsGroup()) {
 		modelTo += after->IsCollapsed() ? 0 : after->GroupSize();
@@ -340,10 +362,6 @@ void MacroTreeModel::MoveItemAfter(const std::shared_ptr<Macro> &item,
 		_macros.insert(std::next(_macros.begin(), macroTo), tmp);
 	}
 	endMoveRows();
-
-	for (auto &m : _macros) {
-		m->ResolveMacroRef();
-	}
 }
 
 static inline int
@@ -368,9 +386,6 @@ void MacroTreeModel::Add(std::shared_ptr<Macro> item)
 	_mt->selectionModel()->clear();
 	_mt->selectionModel()->select(createIndex(idx, 0, nullptr),
 				      QItemSelectionModel::Select);
-	for (auto &m : _macros) {
-		m->ResolveMacroRef();
-	}
 }
 
 void MacroTreeModel::Remove(std::shared_ptr<Macro> item)
@@ -390,6 +405,8 @@ void MacroTreeModel::Remove(std::shared_ptr<Macro> item)
 		if (!item->IsCollapsed()) {
 			uiEndIdx += item->GroupSize();
 		}
+	} else if (item->IsSubitem()) {
+		Macro::PrepareMoveToGroup(nullptr, item);
 	}
 
 	beginRemoveRows(QModelIndex(), uiStartIdx, uiEndIdx);
@@ -402,10 +419,6 @@ void MacroTreeModel::Remove(std::shared_ptr<Macro> item)
 	if (isGroup) {
 		UpdateGroupState(true);
 	}
-
-	for (auto &m : _macros) {
-		m->ResolveMacroRef();
-	}
 }
 
 std::shared_ptr<Macro> MacroTreeModel::Neighbor(const std::shared_ptr<Macro> &m,
@@ -472,6 +485,21 @@ std::shared_ptr<Macro> MacroTreeModel::GetCurrentMacro() const
 	return _macros[ModelIndexToMacroIndex(idx, _macros)];
 }
 
+std::vector<std::shared_ptr<Macro>>
+MacroTreeModel::GetCurrentMacros(const QModelIndexList &selection) const
+{
+	std::vector<std::shared_ptr<Macro>> result;
+	result.reserve(selection.size());
+	for (const auto &sel : selection) {
+		try {
+			result.emplace_back(_macros.at(
+				ModelIndexToMacroIndex(sel.row(), _macros)));
+		} catch (const std::out_of_range &) {
+		}
+	}
+	return result;
+}
+
 MacroTreeModel::MacroTreeModel(MacroTree *st_,
 			       std::deque<std::shared_ptr<Macro>> &macros)
 	: QAbstractListModel(st_), _mt(st_), _macros(macros)
@@ -566,40 +594,42 @@ void MacroTreeModel::GroupSelectedItems(QModelIndexList &indices)
 
 	std::lock_guard<std::mutex> lock(switcher->m);
 	QString name = GetNewGroupName();
-	std::vector<std::shared_ptr<Macro>> items;
-	items.reserve(indices.size());
+	std::vector<std::shared_ptr<Macro>> itemsToGroup;
+	itemsToGroup.reserve(indices.size());
+	int insertGroupAt = indices[0].row();
 	for (int i = 0; i < indices.count(); i++) {
-		std::shared_ptr<Macro> item = _macros[ModelIndexToMacroIndex(
-			indices[i].row(), _macros)];
-		items.emplace_back(item);
+		int selectedIdx = indices[i].row();
+		if (selectedIdx < insertGroupAt) {
+			insertGroupAt = selectedIdx;
+		}
+
+		std::shared_ptr<Macro> item =
+			_macros[ModelIndexToMacroIndex(selectedIdx, _macros)];
+		itemsToGroup.emplace_back(item);
 	}
+
 	std::shared_ptr<Macro> item =
-		Macro::CreateGroup(name.toStdString(), items);
+		Macro::CreateGroup(name.toStdString(), itemsToGroup);
 	if (!item) {
 		return;
 	}
 
 	// A new list entry for group
-	Add(item);
+	_macros.insert(_macros.begin() + insertGroupAt, item);
 
 	// Move all selected items after new group entry
-	for (const auto &item : items) {
+	int offset = 1;
+	for (const auto &item : itemsToGroup) {
 		auto it = _macros.begin() + GetItemMacroIndex(item);
-		std::rotate(it, std::next(it, 1), _macros.end());
+		_macros.erase(it);
+		_macros.insert(_macros.begin() + insertGroupAt + offset, item);
+		offset++;
 	}
 
 	_hasGroups = true;
 	_mt->selectionModel()->clear();
 
-	for (auto &m : _macros) {
-		m->ResolveMacroRef();
-	}
-
 	Reset(_macros);
-
-	for (auto &m : _macros) {
-		m->ResolveMacroRef();
-	}
 }
 
 void MacroTreeModel::UngroupSelectedGroups(QModelIndexList &indices)
@@ -619,15 +649,7 @@ void MacroTreeModel::UngroupSelectedGroups(QModelIndexList &indices)
 
 	_mt->selectionModel()->clear();
 
-	for (auto &m : _macros) {
-		m->ResolveMacroRef();
-	}
-
 	Reset(_macros);
-
-	for (auto &m : _macros) {
-		m->ResolveMacroRef();
-	}
 }
 
 void MacroTreeModel::ExpandGroup(std::shared_ptr<Macro> item)
@@ -642,10 +664,6 @@ void MacroTreeModel::ExpandGroup(std::shared_ptr<Macro> item)
 	Reset(_macros);
 
 	_mt->selectionModel()->clear();
-
-	for (auto &m : _macros) {
-		m->ResolveMacroRef();
-	}
 }
 
 void MacroTreeModel::CollapseGroup(std::shared_ptr<Macro> item)
@@ -660,10 +678,6 @@ void MacroTreeModel::CollapseGroup(std::shared_ptr<Macro> item)
 	Reset(_macros);
 
 	_mt->selectionModel()->clear();
-
-	for (auto &m : _macros) {
-		m->ResolveMacroRef();
-	}
 }
 
 void MacroTreeModel::UpdateGroupState(bool update)
@@ -693,9 +707,13 @@ void MacroTree::Reset(std::deque<std::shared_ptr<Macro>> &macros,
 	GetModel()->Reset(macros);
 }
 
-void MacroTree::Add(std::shared_ptr<Macro> item) const
+void MacroTree::Add(std::shared_ptr<Macro> item,
+		    std::shared_ptr<Macro> after) const
 {
 	GetModel()->Add(item);
+	if (after) {
+		MoveItemAfter(item, after);
+	}
 }
 
 std::shared_ptr<Macro> MacroTree::GetCurrentMacro() const
@@ -703,6 +721,12 @@ std::shared_ptr<Macro> MacroTree::GetCurrentMacro() const
 	return GetModel()->GetCurrentMacro();
 }
 
+std::vector<std::shared_ptr<Macro>> MacroTree::GetCurrentMacros() const
+{
+	QModelIndexList indices = selectedIndexes();
+	return GetModel()->GetCurrentMacros(indices);
+}
+
 MacroTree::MacroTree(QWidget *parent_) : QListView(parent_)
 {
 	setStyleSheet(QString(
@@ -763,8 +787,6 @@ void MacroTree::UpdateWidgets(bool force)
 	}
 }
 
-Q_DECLARE_METATYPE(std::shared_ptr<Macro>);
-
 static inline void MoveItem(std::deque<std::shared_ptr<Macro>> &items,
 			    std::shared_ptr<Macro> &item, int to)
 {
@@ -817,7 +839,7 @@ void MacroTree::dropEvent(QDropEvent *event)
 	Macro *dropItem = items[ModelIndexToMacroIndex(row, items)]
 				  .get(); // Item being dropped on
 	bool itemIsGroup = dropItem->IsGroup();
-	Macro *dropGroup = itemIsGroup ? dropItem : dropItem->Parent();
+	Macro *dropGroup = itemIsGroup ? dropItem : dropItem->Parent().get();
 
 	// Not a group if moving above the group
 	if (indicator == QAbstractItemView::AboveItem && itemIsGroup) {
@@ -863,7 +885,7 @@ void MacroTree::dropEvent(QDropEvent *event)
 	}
 
 	if (hasGroups) {
-		if (!itemBelow || itemBelow->Parent() != dropGroup) {
+		if (!itemBelow || itemBelow->Parent().get() != dropGroup) {
 			dropGroup = nullptr;
 			dropOnCollapsed = false;
 		}
@@ -906,7 +928,7 @@ void MacroTree::dropEvent(QDropEvent *event)
 			bool collapsed = item->IsCollapsed();
 			for (int j = items.size() - 1; j >= 0; j--) {
 				std::shared_ptr<Macro> subitem = items[j];
-				auto subitemGroup = subitem->Parent();
+				auto subitemGroup = subitem->Parent().get();
 
 				if (subitemGroup == item.get()) {
 					if (!collapsed) {
@@ -1015,10 +1037,6 @@ void MacroTree::dropEvent(QDropEvent *event)
 		items.insert(std::next(it, 1), i);
 	}
 
-	for (auto &m : items) {
-		m->ResolveMacroRef();
-	}
-
 	// Update widgets and accept event
 	UpdateWidgets(true);
 	event->accept();
@@ -1029,14 +1047,12 @@ void MacroTree::dropEvent(QDropEvent *event)
 
 bool MacroTree::GroupsSelected() const
 {
-	MacroTreeModel *mtm = GetModel();
-	QModelIndexList selectedIndices = selectedIndexes();
-
 	if (SelectionEmpty()) {
 		return false;
 	}
 
-	for (auto &idx : selectedIndices) {
+	MacroTreeModel *mtm = GetModel();
+	for (auto &idx : selectedIndexes()) {
 		std::shared_ptr<Macro> item =
 			mtm->_macros[ModelIndexToMacroIndex(idx.row(),
 							    mtm->_macros)];
@@ -1049,14 +1065,12 @@ bool MacroTree::GroupsSelected() const
 
 bool MacroTree::GroupedItemsSelected() const
 {
-	MacroTreeModel *mtm = GetModel();
-	QModelIndexList selectedIndices = selectedIndexes();
-
 	if (SelectionEmpty()) {
 		return false;
 	}
 
-	for (auto &idx : selectedIndices) {
+	auto *mtm = GetModel();
+	for (auto &idx : selectedIndexes()) {
 		std::shared_ptr<Macro> item =
 			mtm->_macros[ModelIndexToMacroIndex(idx.row(),
 							    mtm->_macros)];
@@ -1068,6 +1082,11 @@ bool MacroTree::GroupedItemsSelected() const
 	return false;
 }
 
+bool MacroTree::SingleItemSelected() const
+{
+	return selectedIndexes().size() == 1;
+}
+
 bool MacroTree::SelectionEmpty() const
 {
 	return selectedIndexes().empty();
@@ -1208,3 +1227,5 @@ QSize MacroTreeDelegate::sizeHint(const QStyleOptionViewItem &option,
 
 	return (QSize(option.widget->minimumWidth(), item->height()));
 }
+
+} // namespace advss
