@@ -1,14 +1,18 @@
 #include "macro.hpp"
 #include "macro-action-edit.hpp"
 #include "macro-condition-edit.hpp"
+#include "macro-dock.hpp"
 #include "macro-action-scene-switch.hpp"
-#include "advanced-scene-switcher.hpp"
+#include "switcher-data.hpp"
 #include "hotkey.hpp"
 
 #include <limits>
 #undef max
 #include <chrono>
 #include <unordered_map>
+#include <QMainWindow>
+
+namespace advss {
 
 constexpr int perfLogThreshold = 300;
 
@@ -26,6 +30,7 @@ Macro::~Macro()
 	_die = true;
 	Stop();
 	ClearHotkeys();
+	RemoveDock();
 }
 
 std::shared_ptr<Macro>
@@ -211,7 +216,7 @@ bool Macro::PerformActions(bool forceParallel, bool ignorePause)
 	return ret;
 }
 
-int64_t Macro::MsSinceLastCheck()
+int64_t Macro::MsSinceLastCheck() const
 {
 	if (_lastCheckTime.time_since_epoch().count() == 0) {
 		return 0;
@@ -227,6 +232,7 @@ void Macro::SetName(const std::string &name)
 {
 	_name = name;
 	SetHotkeysDesc();
+	SetDockWidgetName();
 }
 
 void Macro::ResetTimers()
@@ -263,6 +269,11 @@ void Macro::SetOnChangeHighlight()
 	_onChangeTriggered = true;
 }
 
+bool Macro::DockIsVisible() const
+{
+	return _dock && _dockAction && _dock->isVisible();
+}
+
 void Macro::SetPaused(bool pause)
 {
 	if (_paused && !pause) {
@@ -296,6 +307,16 @@ void Macro::Stop()
 	}
 }
 
+std::deque<std::shared_ptr<MacroCondition>> &Macro::Conditions()
+{
+	return _conditions;
+}
+
+std::deque<std::shared_ptr<MacroAction>> &Macro::Actions()
+{
+	return _actions;
+}
+
 void Macro::UpdateActionIndices()
 {
 	int idx = 0;
@@ -314,10 +335,9 @@ void Macro::UpdateConditionIndices()
 	}
 }
 
-Macro *Macro::Parent()
+std::shared_ptr<Macro> Macro::Parent() const
 {
-	auto p = _parent.lock();
-	return p.get();
+	return _parent.lock();
 }
 
 bool Macro::Save(obs_data_t *obj) const
@@ -337,6 +357,8 @@ bool Macro::Save(obs_data_t *obj) const
 		return true;
 	}
 
+	SaveDockSettings(obj);
+
 	obs_data_set_bool(obj, "registerHotkeys", _registerHotkeys);
 	obs_data_array_t *pauseHotkey = obs_hotkey_save(_pauseHotkey);
 	obs_data_set_array(obj, "pauseHotkey", pauseHotkey);
@@ -426,6 +448,8 @@ bool Macro::Load(obs_data_t *obj)
 		return true;
 	}
 
+	LoadDockSettings(obj);
+
 	obs_data_set_default_bool(obj, "registerHotkeys", true);
 	_registerHotkeys = obs_data_get_bool(obj, "registerHotkeys");
 	if (_registerHotkeys) {
@@ -498,7 +522,6 @@ bool Macro::Load(obs_data_t *obj)
 
 bool Macro::PostLoad()
 {
-	ResolveMacroRef();
 	for (auto &c : _conditions) {
 		c->PostLoad();
 	}
@@ -508,34 +531,7 @@ bool Macro::PostLoad()
 	return true;
 }
 
-void Macro::ResolveMacroRef()
-{
-	for (auto &c : _conditions) {
-		MacroRefCondition *ref =
-			dynamic_cast<MacroRefCondition *>(c.get());
-		if (ref) {
-			ref->ResolveMacroRef();
-		}
-		MultiMacroRefCondtition *ref2 =
-			dynamic_cast<MultiMacroRefCondtition *>(c.get());
-		if (ref2) {
-			ref2->ResolveMacroRef();
-		}
-	}
-	for (auto &a : _actions) {
-		MacroRefAction *ref = dynamic_cast<MacroRefAction *>(a.get());
-		if (ref) {
-			ref->ResolveMacroRef();
-		}
-		MultiMacroRefAction *ref2 =
-			dynamic_cast<MultiMacroRefAction *>(a.get());
-		if (ref2) {
-			ref2->ResolveMacroRef();
-		}
-	}
-}
-
-bool Macro::SwitchesScene()
+bool Macro::SwitchesScene() const
 {
 	MacroActionSwitchScene temp(nullptr);
 	auto sceneSwitchId = temp.GetId();
@@ -590,11 +586,207 @@ void Macro::EnablePauseHotkeys(bool value)
 	_registerHotkeys = value;
 }
 
-bool Macro::PauseHotkeysEnabled()
+bool Macro::PauseHotkeysEnabled() const
 {
 	return _registerHotkeys;
 }
 
+void Macro::SaveDockSettings(obs_data_t *obj) const
+{
+	auto dockSettings = obs_data_create();
+	obs_data_set_bool(dockSettings, "register", _registerDock);
+	// The object name is used to restore the position of the dock
+	if (_registerDock) {
+		SetDockWidgetName();
+	}
+	obs_data_set_bool(dockSettings, "hasRunButton", _dockHasRunButton);
+	obs_data_set_bool(dockSettings, "hasPauseButton", _dockHasPauseButton);
+	obs_data_set_string(dockSettings, "runButtonText",
+			    _runButtonText.c_str());
+	obs_data_set_string(dockSettings, "pauseButtonText",
+			    _pauseButtonText.c_str());
+	obs_data_set_string(dockSettings, "unpauseButtonText",
+			    _unpauseButtonText.c_str());
+	if (_dock) {
+		auto window = static_cast<QMainWindow *>(
+			obs_frontend_get_main_window());
+		obs_data_set_bool(dockSettings, "isFloating",
+				  _dock->isFloating());
+		obs_data_set_bool(dockSettings, "isVisible", DockIsVisible());
+		obs_data_set_int(dockSettings, "area",
+				 window->dockWidgetArea(_dock));
+		obs_data_set_string(
+			dockSettings, "geometry",
+			_dock->saveGeometry().toBase64().constData());
+	}
+	obs_data_set_obj(obj, "dockSettings", dockSettings);
+	obs_data_release(dockSettings);
+}
+
+void Macro::LoadDockSettings(obs_data_t *obj)
+{
+	auto dockSettings = obs_data_get_obj(obj, "dockSettings");
+	if (!dockSettings) {
+		// TODO: Remove this fallback
+		_dockHasRunButton = obs_data_get_bool(obj, "dockHasRunButton");
+		_dockHasPauseButton =
+			obs_data_get_bool(obj, "dockHasPauseButton");
+		EnableDock(obs_data_get_bool(obj, "registerDock"));
+		return;
+	}
+
+	const bool dockEnabled = obs_data_get_bool(dockSettings, "register");
+	_dockIsVisible = obs_data_get_bool(dockSettings, "isVisible");
+
+	// TODO: remove these default settings in a future version
+	obs_data_set_default_string(
+		dockSettings, "runButtonText",
+		obs_module_text("AdvSceneSwitcher.macroDock.run"));
+	obs_data_set_default_string(
+		dockSettings, "pauseButtonText",
+		obs_module_text("AdvSceneSwitcher.macroDock.pause"));
+	obs_data_set_default_string(
+		dockSettings, "unpauseButtonText",
+		obs_module_text("AdvSceneSwitcher.macroDock.unpause"));
+
+	_runButtonText = obs_data_get_string(dockSettings, "runButtonText");
+	_pauseButtonText = obs_data_get_string(dockSettings, "pauseButtonText");
+	_unpauseButtonText =
+		obs_data_get_string(dockSettings, "unpauseButtonText");
+	if (dockEnabled) {
+		_dockHasRunButton =
+			obs_data_get_bool(dockSettings, "hasRunButton");
+		_dockHasPauseButton =
+			obs_data_get_bool(dockSettings, "hasPauseButton");
+		_dockIsFloating = obs_data_get_bool(dockSettings, "isFloating");
+		_dockArea = static_cast<Qt::DockWidgetArea>(
+			obs_data_get_int(dockSettings, "area"));
+		auto geometryStr =
+			obs_data_get_string(dockSettings, "geometry");
+		if (geometryStr && strlen(geometryStr)) {
+			_dockGeo =
+				QByteArray::fromBase64(QByteArray(geometryStr));
+		}
+	}
+	EnableDock(dockEnabled);
+	obs_data_release(dockSettings);
+}
+
+void Macro::EnableDock(bool value)
+{
+	if (_registerDock == value) {
+		return;
+	}
+
+	// Reset dock regardless
+	RemoveDock();
+
+	// Unregister dock
+	if (_registerDock) {
+		_dockIsFloating = true;
+		_dockGeo = QByteArray();
+		_registerDock = value;
+		return;
+	}
+
+	// Create new dock widget
+	auto window =
+		static_cast<QMainWindow *>(obs_frontend_get_main_window());
+	_dock = new MacroDock(this, window,
+			      QString::fromStdString(_runButtonText),
+			      QString::fromStdString(_pauseButtonText),
+			      QString::fromStdString(_unpauseButtonText));
+	SetDockWidgetName(); // Used by OBS to restore position
+
+	// Register new dock
+	_dockAction = static_cast<QAction *>(obs_frontend_add_dock(_dock));
+
+	// Note that OBSBasic::OBSInit() has precedence over the visibility and
+	// geometry set here.
+	// The function calls here are only intended to attempt to restore the
+	// dock status when switching scene collections.
+	_dock->setVisible(_dockIsVisible);
+	if (window->dockWidgetArea(_dock) != _dockArea) {
+		window->addDockWidget(_dockArea, _dock);
+	}
+	if (_dock->isFloating() != _dockIsFloating) {
+		_dock->setFloating(_dockIsFloating);
+	}
+	_dock->restoreGeometry(_dockGeo);
+	_registerDock = value;
+}
+
+void Macro::SetDockHasRunButton(bool value)
+{
+	_dockHasRunButton = value;
+	if (!_dock) {
+		return;
+	}
+	_dock->ShowRunButton(value);
+}
+
+void Macro::SetDockHasPauseButton(bool value)
+{
+	_dockHasPauseButton = value;
+	if (!_dock) {
+		return;
+	}
+	_dock->ShowPauseButton(value);
+}
+
+void Macro::SetRunButtonText(const std::string &text)
+{
+	_runButtonText = text;
+	if (!_dock) {
+		return;
+	}
+	_dock->SetRunButtonText(QString::fromStdString(text));
+}
+
+void Macro::SetPauseButtonText(const std::string &text)
+{
+	_pauseButtonText = text;
+	if (!_dock) {
+		return;
+	}
+	_dock->SetPauseButtonText(QString::fromStdString(text));
+}
+
+void Macro::SetUnpauseButtonText(const std::string &text)
+{
+	_unpauseButtonText = text;
+	if (!_dock) {
+		return;
+	}
+	_dock->SetUnpauseButtonText(QString::fromStdString(text));
+}
+
+void Macro::RemoveDock()
+{
+	if (_dock) {
+		_dock->close();
+		_dock->deleteLater();
+		_dockAction->deleteLater();
+		_dock = nullptr;
+		_dockAction = nullptr;
+	}
+}
+
+void Macro::SetDockWidgetName() const
+{
+	if (!_dock) {
+		return;
+	}
+	// Set prefix to avoid dock name conflict
+	_dock->setObjectName("ADVSS-" + QString::fromStdString(_name));
+	_dock->SetName(QString::fromStdString(_name));
+
+	if (!_dockAction) {
+		return;
+	}
+	_dockAction->setText(QString::fromStdString(_name));
+}
+
 static void pauseCB(void *data, obs_hotkey_id, obs_hotkey_t *, bool pressed)
 {
 	if (pressed) {
@@ -622,9 +814,9 @@ static void togglePauseCB(void *data, obs_hotkey_id, obs_hotkey_t *,
 
 static int macroHotkeyID = 0;
 
-obs_hotkey_id registerHotkeyHelper(const std::string prefix,
-				   const char *formatModuleText, Macro *macro,
-				   obs_hotkey_func func)
+static obs_hotkey_id registerHotkeyHelper(const std::string prefix,
+					  const char *formatModuleText,
+					  Macro *macro, obs_hotkey_func func)
 {
 	macroHotkeyID++;
 
@@ -656,7 +848,7 @@ void Macro::SetupHotkeys()
 		togglePauseCB);
 }
 
-void Macro::ClearHotkeys()
+void Macro::ClearHotkeys() const
 {
 	obs_hotkey_unregister(_pauseHotkey);
 	obs_hotkey_unregister(_unpauseHotkey);
@@ -671,7 +863,7 @@ void setHotkeyDescriptionHelper(const char *formatModuleText,
 	obs_hotkey_set_description(id, hotkeyDesc.toStdString().c_str());
 }
 
-void Macro::SetHotkeysDesc()
+void Macro::SetHotkeysDesc() const
 {
 	setHotkeyDescriptionHelper("AdvSceneSwitcher.hotkey.macro.pause", _name,
 				   _pauseHotkey);
@@ -681,7 +873,7 @@ void Macro::SetHotkeysDesc()
 				   _name, _togglePauseHotkey);
 }
 
-void SwitcherData::saveMacros(obs_data_t *obj)
+void SwitcherData::SaveMacros(obs_data_t *obj)
 {
 	switcher->macroProperties.Save(obj);
 
@@ -698,7 +890,7 @@ void SwitcherData::saveMacros(obs_data_t *obj)
 	obs_data_array_release(macroArray);
 }
 
-void SwitcherData::loadMacros(obs_data_t *obj)
+void SwitcherData::LoadMacros(obs_data_t *obj)
 {
 	Hotkey::ClearAllHotkeys();
 	switcher->macroProperties.Load(obj);
@@ -753,7 +945,7 @@ void SwitcherData::loadMacros(obs_data_t *obj)
 	}
 }
 
-bool SwitcherData::checkMacros()
+bool SwitcherData::CheckMacros()
 {
 	bool ret = false;
 	for (auto &m : macros) {
@@ -769,7 +961,7 @@ bool SwitcherData::checkMacros()
 	return ret;
 }
 
-bool SwitcherData::runMacros()
+bool SwitcherData::RunMacros()
 {
 	// Create copy of macor list as elements might be removed, inserted, or
 	// reordered while macros are currently being executed.
@@ -821,3 +1013,16 @@ Macro *GetMacroByQString(const QString &name)
 {
 	return GetMacroByName(name.toUtf8().constData());
 }
+
+std::weak_ptr<Macro> GetWeakMacroByName(const char *name)
+{
+	for (auto &m : switcher->macros) {
+		if (m->Name() == name) {
+			return m;
+		}
+	}
+
+	return {};
+}
+
+} // namespace advss
