@@ -1,5 +1,7 @@
 #include "transition-selection.hpp"
-#include "advanced-scene-switcher.hpp"
+#include "switcher-data.hpp"
+
+namespace advss {
 
 void TransitionSelection::Save(obs_data_t *obj, const char *name,
 			       const char *typeName) const
@@ -68,7 +70,7 @@ TransitionSelectionWidget::TransitionSelectionWidget(QWidget *parent,
 	: QComboBox(parent)
 {
 	setDuplicatesEnabled(true);
-	populateTransitionSelection(this, current, any);
+	PopulateTransitionSelection(this, current, any);
 
 	QWidget::connect(this, SIGNAL(currentTextChanged(const QString &)),
 			 this, SLOT(SelectionChanged(const QString &)));
@@ -112,7 +114,7 @@ void TransitionSelectionWidget::Repopulate(bool current, bool any)
 	{
 		const QSignalBlocker blocker(this);
 		clear();
-		populateTransitionSelection(this, current, any);
+		PopulateTransitionSelection(this, current, any);
 		setCurrentIndex(0);
 	}
 	TransitionSelection t;
@@ -169,3 +171,5 @@ void TransitionSelectionWidget::SelectionChanged(const QString &name)
 
 	emit TransitionChanged(t);
 }
+
+} // namespace advss
