@@ -1,11 +1,19 @@
 #include "variable.hpp"
+#include "switcher-data.hpp"
+#include "math-helpers.hpp"
+#include "utility.hpp"
 
-#include <switcher-data-structs.hpp>
+namespace advss {
 
 // Keep track of the last time a variable was changed to save some work when
-// when resolving strings containing variables
+// when resolving strings containing variables, etc.
 static std::chrono::high_resolution_clock::time_point lastVariableChange{};
 
+std::chrono::high_resolution_clock::time_point GetLastVariableChangeTime()
+{
+	return lastVariableChange;
+}
+
 Variable::Variable() : Item()
 {
 	lastVariableChange = std::chrono::high_resolution_clock::now();
@@ -40,11 +48,9 @@ void Variable::Save(obs_data_t *obj) const
 	obs_data_set_string(obj, "defaultValue", _defaultValue.c_str());
 }
 
-bool Variable::DoubleValue(double &value) const
+std::optional<double> Variable::DoubleValue() const
 {
-	char *end = nullptr;
-	value = strtod(_value.c_str(), &end);
-	return end != _value.c_str() && *end == '\0' && value != HUGE_VAL;
+	return GetDouble(_value);
 }
 
 void Variable::SetValue(const std::string &val)
@@ -101,84 +107,16 @@ QStringList GetVariablesNameList()
 	return list;
 }
 
-void VariableResolvingString::Resolve()
+std::string GetWeakVariableName(std::weak_ptr<Variable> var_)
 {
-	if (switcher->variables.empty()) {
-		_resolvedValue = _value;
-		return;
+	auto var = var_.lock();
+	if (!var) {
+		return "invalid variable selection";
 	}
-	if (_lastResolve == lastVariableChange) {
-		return;
-	}
-	_resolvedValue = SubstitueVariables(_value);
-	_lastResolve = lastVariableChange;
+	return var->Name();
 }
 
-VariableResolvingString::operator std::string()
-{
-	Resolve();
-	return _resolvedValue;
-}
-
-void VariableResolvingString::operator=(std::string value)
-{
-	_value = value;
-	_lastResolve = {};
-}
-
-void VariableResolvingString::operator=(const char *value)
-{
-	_value = value;
-	_lastResolve = {};
-}
-
-void VariableResolvingString::Load(obs_data_t *obj, const char *name)
-{
-	_value = obs_data_get_string(obj, name);
-	Resolve();
-}
-
-void VariableResolvingString::Save(obs_data_t *obj, const char *name) const
-{
-	obs_data_set_string(obj, name, _value.c_str());
-}
-
-const char *VariableResolvingString::c_str()
-{
-	Resolve();
-	return _resolvedValue.c_str();
-}
-
-const char *VariableResolvingString::c_str() const
-{
-	// Just assume that the value was previously resolved already
-	return _resolvedValue.c_str();
-}
-
-static void replaceAll(std::string &str, const std::string &from,
-		       const std::string &to)
-{
-	if (from.empty()) {
-		return;
-	}
-	size_t start_pos = 0;
-	while ((start_pos = str.find(from, start_pos)) != std::string::npos) {
-		str.replace(start_pos, from.length(), to);
-		start_pos += to.length();
-	}
-}
-
-std::string SubstitueVariables(std::string str)
-{
-	for (const auto &v : switcher->variables) {
-		const auto &variable = std::dynamic_pointer_cast<Variable>(v);
-		const std::string pattern = "${" + variable->Name() + "}";
-		replaceAll(str, pattern, variable->Value());
-	}
-	return str;
-}
-
-void SwitcherData::saveVariables(obs_data_t *obj)
+void SwitcherData::SaveVariables(obs_data_t *obj)
 {
 	obs_data_array_t *variablesArray = obs_data_array_create();
 	for (const auto &v : variables) {
@@ -191,7 +129,7 @@ void SwitcherData::saveVariables(obs_data_t *obj)
 	obs_data_array_release(variablesArray);
 }
 
-void SwitcherData::loadVariables(obs_data_t *obj)
+void SwitcherData::LoadVariables(obs_data_t *obj)
 {
 	variables.clear();
 
@@ -336,3 +274,16 @@ void VariableSelection::SetVariable(const std::string &variable)
 		_selection->setCurrentIndex(0);
 	}
 }
+
+void VariableSelection::SetVariable(const std::weak_ptr<Variable> &variable_)
+{
+	const QSignalBlocker blocker(_selection);
+	auto var = variable_.lock();
+	if (var) {
+		SetVariable(var->Name());
+	} else {
+		_selection->setCurrentIndex(0);
+	}
+}
+
+} // namespace advss
